
#include "stdafx.h"
#include "editcmd.h"
#include "resource.h"
#include "prjtreec.h"
#include "sheetpls.h"
#include "registry.h"
#include "editpad.h"
#include "wm.h"
#include "filesup.h"
#include "edfopdlg.h"
#include "edfaddlg.h"
	
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define ICON_WIDTH 16
#define ICON_HEIGHT 16
#define STATUS_WIDTH ICON_WIDTH
#define STATUS_HEIGHT ICON_HEIGHT
#define TEXT_DELTA 2

#pragma warning ( disable : 4711 )
/////////////////////////////////////////////////////////////////////////////
// CProjectTreeCtrl

CProjectTreeCtrl::CProjectTreeCtrl () : m_dwLayout (0)
{
}

CProjectTreeCtrl:: ~ CProjectTreeCtrl ()
{
}

BOOL CProjectTreeCtrl::ExpandItem (HTREEITEM hItem, UINT nCode)
{
  if (Expand (hItem, nCode))
    {
      int iNormal, iSelected;
      if (GetItemState (hItem, TVIS_EXPANDED) & TVIS_EXPANDED)
        {
          if (GetItemImage (hItem, iNormal, iSelected) && iNormal == 2)
            SetItemImage (hItem, 3, 3);
        }
      else
        {
          if (GetItemImage (hItem, iNormal, iSelected) && iNormal == 3)
            SetItemImage (hItem, 2, 2);
        }
      return TRUE;
    }
  return FALSE;
}

HTREEITEM CProjectTreeCtrl::InsertFile (LPCTSTR lpszItem, int nImage, int nSelectedImage, LPARAM lParam, HTREEITEM hParent)
{
  HTREEITEM hItem = GetNextItem (hParent, TVGN_CHILD), hAfter = TVI_FIRST;
  CProjectItem *pItem;
  while (hItem)
    {
      pItem = (CProjectItem*) GetItemData (hItem);
      if (pItem->dwType & PI_FILE)
        break;
      hAfter = hItem;
      hItem = GetNextItem (hItem, TVGN_NEXT);
    }
  while (hItem)
    {
      pItem = (CProjectItem*) GetItemData (hItem);
      if (_tcsicmp (GetItemText (hItem), lpszItem) > 0)
        break;
      hAfter = hItem;
      hItem = GetNextItem (hItem, TVGN_NEXT);
    }
  if (!hItem)
    hAfter = TVI_LAST;
  return InsertItem (TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE|TVIF_PARAM, lpszItem, nImage, nSelectedImage, 0, 0, lParam, hParent, hAfter);
}

HTREEITEM CProjectTreeCtrl::InsertGroup (LPCTSTR lpszItem, int nImage, int nSelectedImage, LPARAM lParam, HTREEITEM hParent)
{
  HTREEITEM hItem = GetNextItem (hParent, TVGN_CHILD), hAfter = TVI_FIRST;
  CProjectItem *pItem;
  while (hItem)
    {
      pItem = (CProjectItem*) GetItemData (hItem);
      if ((pItem->dwType & PI_FILE) || _tcsicmp (GetItemText (hItem), lpszItem) > 0)
        break;
      hAfter = hItem;
      hItem = GetNextItem (hItem, TVGN_NEXT);
    }
  return InsertItem (TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE|TVIF_PARAM, lpszItem, nImage, nSelectedImage, 0, 0, lParam, hParent, hAfter);
}

BOOL CProjectTreeCtrl::
Create (DWORD dwStyle, CRect rect, CWnd * pParentWnd, UINT nID)
{
  if (m_dwLayout & PRJTREEC_HEADER)
    {
      rect.top += 15;
      rect.bottom -= 15;
    }
  return CAppTreeCtrl::Create (dwStyle, rect, pParentWnd, nID);
}

static int GetItemIcon (LPCTSTR pszName)
{
  int iIcon;
  CString sExt = GetNameExt (pszName);
  sExt.MakeLower ();
  if (sExt == _T ("c"))
    iIcon = 6;
  else if (sExt == _T ("cc") || sExt == _T ("cpp") || sExt == _T ("cxx"))
    iIcon = 7;
  else if (sExt == _T ("def"))
    iIcon = 8;
  else if (sExt == _T ("h") || sExt == _T ("hpp") || sExt == _T ("hxx") || sExt == _T ("inl") || sExt == _T ("hm"))
    iIcon = 9;
  else if (sExt == _T ("rc") || sExt == _T ("rc2"))
    iIcon = 10;
  else if (sExt == _T ("ico") || sExt == _T ("bmp") || sExt == _T ("cur") || sExt == _T ("gif") || sExt == _T ("jpg"))
    iIcon = 5;
  else
    iIcon = 4;
  return iIcon;
}

void CProjectTreeCtrl::
PopulateItem (CProjectItem *pParent, HTREEITEM hParent)
{
  CString sName;
  CProjectItem *pItem;
  POSITION pos = pParent->lstItems.GetHeadPosition ();
  int iIcon;
  while (pos)
    {
      pItem = pParent->lstItems.GetNext (pos);
      sName = GetNameExt (pItem->sPath);
      if (sName.IsEmpty ())
        sName = _T ("Unnamed");
      if (pItem->dwType & PI_FILE)
        {
          iIcon = GetItemIcon (pItem->sPath);
          InsertFile (sName, iIcon, iIcon, (LPARAM) pItem, hParent);
        }
      else
        PopulateItem (pItem, InsertGroup (sName, 2, 2, (LPARAM) pItem, hParent));
    }
}

void CProjectTreeCtrl::
Populate ()
{
  PopulateEmpty ();
  if (!g_pProject)
    return;
  CString sName = GetName (g_pProject->sPath);
  if (sName.IsEmpty ())
    sName = _T ("Unnamed");
  HTREEITEM hParent = InsertItem (TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE|TVIF_PARAM, sName, 0, 0, 0, 0, NULL, TVI_ROOT, TVI_FIRST), hItem;
  CProjectItem *pItem;
  int iIcon;
  POSITION pos = g_pProject->lstItems.GetHeadPosition ();
  while (pos)
    {
      pItem = g_pProject->lstItems.GetNext (pos);
      sName = GetNameExt (pItem->sPath);
      if (sName.IsEmpty ())
        sName = _T ("Unnamed");
      if (pItem->dwType & PI_ROOT)
        if (pItem->dwType & PI_FILE) // pItem->lstItems.IsEmpty ())
          {
            iIcon = GetItemIcon (pItem->sPath);
            InsertFile (sName, iIcon, iIcon, (LPARAM) pItem, hParent);
          }
        else
          if (pItem->dwType & PI_SUBITEM)
            {
              hItem = InsertGroup (sName, 1, 1, (LPARAM) pItem, hParent);
              PopulateItem (pItem, hItem);
              // ExpandItem (hItem, TVE_EXPAND);
            }
          else
            PopulateItem (pItem, InsertGroup (sName, 2, 2, (LPARAM) pItem, hParent));
    }
  ExpandItem (hParent, TVE_EXPAND);
  m_listSelected.AddTail (hParent);
}

void CProjectTreeCtrl::
PopulateEmpty ()
{
  m_listSelected.RemoveAll ();
  DeleteAllItems ();
}

void CProjectTreeCtrl::
PopulateName ()
{
  if (!g_pProject)
    return;
  CString sName = GetName (g_pProject->sPath);
  if (sName.IsEmpty ())
    sName = _T ("Unnamed");
  SetItemText (GetRootItem (), sName);
}

void CProjectTreeCtrl::
Paint (void)
{
  // Invalidate ();
  // UpdateWindow ();
  RedrawWindow ();
}

DWORD CProjectTreeCtrl::
GetLayout ()
{
  return m_dwLayout;
}

void CProjectTreeCtrl::
SetLayout (DWORD dwLayout)
{
  if (m_dwLayout != dwLayout)
    {
      m_dwLayout = dwLayout;
      if (!(m_dwLayout & PRJTREEC_HEADER) && ::IsWindow (m_ctlHeader.GetSafeHwnd ()))
        {
          m_ctlHeader.DestroyWindow ();
        }
      Invalidate ();
    }
}

void CProjectTreeCtrl::
MakeColumns ()
{
  if (m_dwLayout & PRJTREEC_HEADER)
    {
      CRect rect;
      GetWindowRect (rect);
      rect.right -= rect.left;
      rect.bottom -= rect.top;
      rect.left = rect.top =0;
      m_ctlHeader.m_pctlTree = this;
      rect.top = 0;
      rect.bottom = 17;
      if (!m_ctlHeader.Create (WS_CHILD | WS_VISIBLE, rect, GetParent (), GetDlgCtrlID () + 1))
        {
          return;
        }
      m_ctlHeader.FlatHeader ();
      CString sName0 = _T ("Name"), sName1 = _T ("Description");
      LPCTSTR pszName = REGISTRY_ROOT _T ("ProjectTree");
      int nWidth0 = 200,  nWidth1 = 250;
      RegLoadNumber (HKEY_CURRENT_USER, pszName, sName0, (DWORD*) &nWidth0);
      RegLoadNumber (HKEY_CURRENT_USER, pszName, sName1, (DWORD*) &nWidth1);
      HDITEM item;
      item.mask = HDI_TEXT | HDI_FORMAT | HDI_WIDTH;
      item.fmt = HDF_LEFT;
      item.cxy = nWidth0;
      item.pszText = (LPTSTR) (LPCTSTR) sName0;
      m_ctlHeader.InsertItem (0, &item);
      item.cxy = nWidth1;
      item.pszText = (LPTSTR) (LPCTSTR) sName1;
      m_ctlHeader.InsertItem (1, &item);
      // m_ctlHeader.RefreshItemRectCache ();
    }
}

BEGIN_MESSAGE_MAP (CProjectTreeCtrl, CAppTreeCtrl)
//{{AFX_MSG_MAP( CProjectTreeCtrl )
ON_NOTIFY_REFLECT (TVN_BEGINLABELEDIT, OnBeginLabelEdit)
ON_NOTIFY_REFLECT (TVN_ENDLABELEDIT, OnEndLabelEdit)
ON_NOTIFY_REFLECT (TVN_ITEMEXPANDED, OnItemExpanded)
ON_NOTIFY_REFLECT (TVN_SELCHANGED, OnSelectionChanged)
ON_WM_CREATE ()
ON_WM_DESTROY ()
ON_WM_LBUTTONDBLCLK ()
ON_WM_PAINT ()
ON_WM_LBUTTONDOWN ()
ON_WM_LBUTTONUP ()
ON_WM_RBUTTONDOWN ()
ON_WM_RBUTTONUP ()
ON_WM_SIZE ()
ON_MESSAGE (WM_APPEND_MENU, OnAppendMenu)
ON_MESSAGE (WM_PROCESS_MENU, OnProcessMenu)
ON_MESSAGE (WM_APPEND_PROPS, OnAppendProps)
ON_COMMAND(ID_PROPERTIES, OnProperties)
ON_COMMAND(ID_OPEN, OnOpen)
ON_COMMAND(ID_NEW_FILE, OnNewFile)
ON_COMMAND(ID_NEW_GROUP, OnNewGroup)
ON_COMMAND(ID_ADD_FILE, OnAddFile)
ON_COMMAND(ID_REMOVE_ITEM, OnRemoveItem)
ON_COMMAND(ID_ITEM_PROPERTIES, OnItemProperties)
ON_COMMAND(ID_RENAME_ITEM, OnRenameItem)
ON_COMMAND(ID_REFRESH, OnRefresh)
//}}AFX_MSG_MAP
// ON_MESSAGE (TVM_HITTEST, OnHitTest)
// ON_WM_TIMER ()
END_MESSAGE_MAP ()

/////////////////////////////////////////////////////////////////////////////
// CProjectTreeCtrl message handlers

void CProjectTreeCtrl::
OnSelectionChanged (NMHDR * pNMHDR, LRESULT * pResult)
{
  TV_DISPINFO *pTVDispInfo = (TV_DISPINFO *) pNMHDR;
  *pResult = 0;
}

void CProjectTreeCtrl::
OnBeginLabelEdit (NMHDR * pNMHDR, LRESULT * pResult)
{
  TV_DISPINFO *pTVDispInfo = (TV_DISPINFO *) pNMHDR;
  CProjectItem *pItem = (CProjectItem*) GetItemData (pTVDispInfo->item.hItem);
  *pResult = !(pItem && (pItem->dwType & PI_GROUP) && !(pItem->dwType & PI_SUBITEM));
}

void CProjectTreeCtrl::
OnEndLabelEdit (NMHDR * pNMHDR, LRESULT * pResult)
{
  TV_DISPINFO *pTVDispInfo = (TV_DISPINFO *) pNMHDR;
  CProjectItem *pItem = (CProjectItem*) GetItemData (pTVDispInfo->item.hItem);
  if (pItem && (pItem->dwType & PI_GROUP) && !(pItem->dwType & PI_SUBITEM))
    {
      SetItemText (pTVDispInfo->item.hItem, pItem->sPath = pTVDispInfo->item.pszText);
      ASSERT (g_pProject);
      g_pProject->SetModified ();
    }
  CWnd *pWnd = AfxGetMainWnd ();
  pWnd->SetForegroundWindow ();
  pWnd->SetFocus ();
}

void CProjectTreeCtrl::
OnItemExpanded (NMHDR * pNMHDR, LRESULT * pResult)
{
  NM_TREEVIEW *pNMTreeView = (NM_TREEVIEW *) pNMHDR;
  if (GetItemState (pNMTreeView->itemNew.hItem, TVIS_EXPANDED) & TVIS_EXPANDED)
    {
      int iNormal, iSelected;
      if (GetItemImage (pNMTreeView->itemNew.hItem, iNormal, iSelected) && iNormal == 2)
        SetItemImage (pNMTreeView->itemNew.hItem, 3, 3);
    }
  else
    {
      int iNormal, iSelected;
      if (GetItemImage (pNMTreeView->itemNew.hItem, iNormal, iSelected) && iNormal == 3)
        SetItemImage (pNMTreeView->itemNew.hItem, 2, 2);
    }
  *pResult = 0;
}

int CProjectTreeCtrl::
OnCreate (LPCREATESTRUCT lpCreateStruct)
{
  if (CAppTreeCtrl::OnCreate (lpCreateStruct) == -1)
    return -1;
  m_objStatusImages.Create (IDB_PRJTREE, ICON_WIDTH, 0, RGB (0, 255, 0));
  SetImageList (&m_objStatusImages, TVSIL_NORMAL);
  return 0;
}

#pragma warning ( disable : 4100 )
void CProjectTreeCtrl::
OnPaint ()
{
  CPaintDC dc (this);           // device context for painting
  CWnd::DefWindowProc(WM_PAINT, (WPARAM)dc.m_hDC, 0);

  /*HTREEITEM hItem = GetFirstVisibleItem ();
  int n = GetVisibleCount () + 1;
  CRect rc, rc0, rc1;
  CFont *pOldFont = NULL;
  CFont normal;
  LOGFONT lf;
  GetFont ()->GetLogFont (&lf);
  lf.lfWeight = FW_NORMAL;
  normal.CreateFontIndirect (&lf);
  pOldFont = dc.SelectObject (&normal);
  CPen pen1 (PS_SOLID, 0,::GetSysColor (COLOR_BTNSHADOW)), pen2 (PS_SOLID, 0,::GetSysColor (COLOR_BTNFACE)),
  pen3 (PS_SOLID, 0,::GetSysColor (COLOR_WINDOWTEXT)), *pOldPen = dc.SelectObject (&pen1);
  CList < int, int >m_ext;
  CString sText;
  if (m_dwLayout & PRJTREEC_TABBEDCOLS)
    {
      if (m_dwLayout & PRJTREEC_HEADER)
        {
          VERIFY (m_ctlHeader.GetCachedItemRect (0, rc0));
          VERIFY (m_ctlHeader.GetCachedItemRect (1, rc1));
        }
      else
        {
          rc0.SetRect (0, 0, 200, 0);
          rc1.SetRect (200, 0, 550, 0);
        }
    }
  while (hItem && n--)
    {
      CString sItem = GetItemText (hItem);
      VERIFY (GetItemRect (hItem, &rc, TRUE));
      rc.right++;
      if (m_dwLayout & PRJTREEC_TABBEDCOLS)
        {
          if (rc0.right < rc.right + STATUS_WIDTH)
            {
              rc.right = rc0.right - STATUS_WIDTH;
            }
        }
      UINT state = GetItemState (hItem, TVIS_SELECTED | TVIS_EXPANDED);
      if (state & TVIS_SELECTED)
        {
          dc.SetTextColor (::GetSysColor (COLOR_HIGHLIGHTTEXT));
          dc.SetBkColor (::GetSysColor (COLOR_HIGHLIGHT));
        }
      else
        {
          dc.SetTextColor (::GetSysColor (COLOR_WINDOWTEXT));
          dc.SetBkColor (::GetSysColor (COLOR_WINDOW));
        }
      if (!m_ext.IsEmpty ())
        {
          POSITION pos = m_ext.GetHeadPosition ();
          int left;
          dc.SelectObject (&pen2);
          do
            {
              left = m_ext.GetNext (pos);
              if (left != rc.left)
                {
                  dc.MoveTo (left - 10, rc.top);
                  dc.LineTo (left - 10, rc.bottom);
                }
            }
          while (pos);
        }
      if (GetNextItem (hItem, TVGN_CHILD))
        {
          dc.SelectObject (&pen1);
          dc.Rectangle (rc.left - 14, rc.top + 4, rc.left - 5, rc.top + 13);
          dc.SelectObject (&pen2);
          dc.MoveTo (rc.left - 5, rc.top + 8);
          dc.LineTo (rc.left, rc.top + 8);
          dc.MoveTo (rc.left - 10, rc.top);
          dc.LineTo (rc.left - 10, rc.top + 4);
          if (GetNextItem (hItem, TVGN_NEXT))
            {
              dc.MoveTo (rc.left - 10, rc.top + 13);
              dc.LineTo (rc.left - 10, rc.bottom);
              if (!m_ext.Find (rc.left))
                {
                  m_ext.AddTail (rc.left);
                }
            }
          else
            {
              POSITION pos = m_ext.Find (rc.left);
              if (pos)
                {
                  m_ext.RemoveAt (pos);
                }
            }
          dc.SelectObject (&pen3);
          dc.MoveTo (rc.left - 12, rc.top + 8);
          dc.LineTo (rc.left - 7, rc.top + 8);
          if (!(state & TVIS_EXPANDED))
            {
              dc.MoveTo (rc.left - 10, rc.top + 6);
              dc.LineTo (rc.left - 10, rc.top + 11);
            }
        }
      else
        {
          dc.SelectObject (&pen2);
          dc.MoveTo (rc.left - 10, rc.top + 8);
          dc.LineTo (rc.left, rc.top + 8);
          if (GetNextItem (hItem, TVGN_NEXT))
            {
              dc.MoveTo (rc.left - 10, rc.top);
              dc.LineTo (rc.left - 10, rc.bottom);
              if (!m_ext.Find (rc.left))
                {
                  m_ext.AddTail (rc.left);
                }
            }
          else
            {
              dc.MoveTo (rc.left - 10, rc.top);
              dc.LineTo (rc.left - 10, rc.top + 8);
              POSITION pos = m_ext.Find (rc.left);
              if (pos)
                {
                  m_ext.RemoveAt (pos);
                }
            }
        }
      int iImage, iSelectedImage;
      VERIFY (GetItemImage (hItem, iImage, iSelectedImage));
      m_objStatusImages.Draw (&dc, iImage, rc.TopLeft (), ILD_TRANSPARENT);
      rc.left += ICON_WIDTH;
      rc.right += ICON_WIDTH;
      dc.ExtTextOut (rc.left + TEXT_DELTA, rc.top + 1, ETO_OPAQUE, rc, sItem, NULL);
      if (state & TVIS_SELECTED)
        {
          dc.DrawFocusRect (rc);
        }
      if (state & TVIS_SELECTED)
        {
          dc.SetTextColor (::GetSysColor (COLOR_WINDOWTEXT));
          dc.SetBkColor (::GetSysColor (COLOR_WINDOW));
        }

      if (m_dwLayout & PRJTREEC_SHOWDESC)
        {
          LPTSTR str = sText.GetBuffer (_tcslen (_T ("DUMMY")) + 1);
          _tcscpy (str, _T ("DUMMY"));
          _tcslwr (str + 1);
          sText.ReleaseBuffer ();
          if (m_dwLayout & PRJTREEC_TABBEDCOLS)
            {
              rc.left = rc1.left;
              rc.right = rc1.right;
            }
          else
            {
              rc.left = rc.right + TEXT_DELTA;
              rc.right = (long) (rc.left + dc.GetTextExtent (sText).cx + (TEXT_DELTA << 1) + 1);
            }
          dc.ExtTextOut (rc.left + TEXT_DELTA, rc.top + 1, ETO_OPAQUE, rc, sText, NULL);
        }
      hItem = GetNextVisibleItem (hItem);
    }
  dc.SelectObject (pOldPen);
  pen1.DeleteObject ();
  pen2.DeleteObject ();
  pen3.DeleteObject ();
  dc.SelectObject (pOldFont);
  normal.DeleteObject ();*/
}

void CProjectTreeCtrl::
OnLButtonDblClk (UINT nFlags, CPoint point)
{
  /*UINT nFlags2;
  HTREEITEM hItem;
  hItem = HitTest (point, &nFlags2);
  KillTimer (1);
  m_bDblClk = true;
  CAppTreeCtrl::OnLButtonDblClk (nFlags, point);
  OnTimer (1);*/
  if (m_hPointedItem)
    {
      //EditLabel (m_hPointedItem);
      CProjectItem *pItem = (CProjectItem*) GetItemData (m_hPointedItem);
      if (pItem && (pItem->dwType & PI_FILE))
        {
          ASSERT (g_pProject);
          CString sPath = g_pProject->MakePath (pItem);
          AfxGetMainWnd ()->SendMessage (WM_OPEN_HERE, 0, (LPARAM) (LPCTSTR) sPath);
        }
      else
        {
          if (pItem && !pItem->lstItems.IsEmpty ())
            ExpandItem (m_hPointedItem, TVE_TOGGLE);
        }
      CWnd *pWnd = AfxGetMainWnd ();
      pWnd->SetForegroundWindow ();
      pWnd->SetFocus ();
    }
  CWnd *pWnd = AfxGetMainWnd ();
  pWnd->SetForegroundWindow ();
  pWnd->SetFocus ();
}

void CProjectTreeCtrl::
OnLButtonDown (UINT nFlags, CPoint point)
{
  UINT nFlags2;
  m_hPointedItem = HitTest (point, &nFlags2);
  /*KillTimer (1);
  m_bDblClk = false;*/
  if (m_hPointedItem && (nFlags2 & (TVHT_ONITEMICON|TVHT_ONITEMLABEL)))
    {
      if (GetFocus () != this)
        {
          SetFocus ();
        }
      return;
    }
  CAppTreeCtrl::OnLButtonDown (nFlags, point);
}

void CProjectTreeCtrl::
OnLButtonUp (UINT nFlags, CPoint point)
{
  UINT nFlags2;
  HTREEITEM hItem = HitTest (point, &nFlags2);

  if (hItem == m_hPointedItem && (nFlags2 & (TVHT_ONITEMICON | TVHT_ONITEMLABEL)))
    {
      if (GetItemState (hItem, TVIS_SELECTED) & TVIS_SELECTED)
        {
          if (nFlags & MK_SHIFT)
            {
              SelectTo (hItem);
            }
          else if (nFlags & MK_CONTROL)
            {
              SetItemState (hItem, 0, TVIS_SELECTED);
              m_listSelected.RemoveAt (m_listSelected.Find(hItem));
            }
          /*else if (!m_bDblClk)
            {
              SetTimer (1, GetDoubleClickTime (), NULL);
            }*/
        }
      else
        {
          if (nFlags & MK_SHIFT)
            {
              SelectTo (hItem);
            }
          else
            {
              if (nFlags & MK_CONTROL)
                {
                  SetItemState (hItem, TVIS_SELECTED, TVIS_SELECTED);
                  m_listSelected.AddTail (hItem);
                }
              else
                {
                  ClearSelection ();
                  SetItemState (hItem, TVIS_SELECTED, TVIS_SELECTED);
                  m_listSelected.AddTail (hItem);
                }
            }
        }
      CWnd *pWnd = AfxGetMainWnd ();
      pWnd->SetForegroundWindow ();
      pWnd->SetFocus ();
      return;
    }
  CAppTreeCtrl::OnLButtonUp (nFlags, point);
  CWnd *pWnd = AfxGetMainWnd ();
  pWnd->SetForegroundWindow ();
  pWnd->SetFocus ();
}

void CProjectTreeCtrl::
OnDestroy ()
{
  if (m_dwLayout & PRJTREEC_HEADER)
    {
      LPCTSTR pszName = REGISTRY_ROOT _T ("ProjectTree");
      HDITEM item;
      item.mask = HDI_WIDTH;
      if (m_ctlHeader.GetItem (0, &item))
        {
          RegSaveNumber (HKEY_CURRENT_USER, pszName, _T ("Description"), item.cxy);
        }
      if (m_ctlHeader.GetItem (1, &item))
        {
          RegSaveNumber (HKEY_CURRENT_USER, pszName, _T ("Name"), item.cxy);
        }
      m_ctlHeader.DestroyWindow ();
    }
  CAppTreeCtrl::OnDestroy ();
}

/*LONG CProjectTreeCtrl::
OnHitTest (WPARAM wParam, LPARAM lParam)
{
  LONG lResult = DefWindowProc (TVM_HITTEST, wParam, lParam);
  LPTVHITTESTINFO pHitTestInfo = (LPTVHITTESTINFO) lParam;

  if (pHitTestInfo->hItem)
    {
      CClientDC dc (this);
      CRect rc;
      GetItemRect (pHitTestInfo->hItem, &rc, TRUE);
      rc.right++;
      UINT state = GetItemState (pHitTestInfo->hItem, TVIS_SELECTED);
      pHitTestInfo->flags = 0;
      if (pHitTestInfo->pt.x > rc.left - ICON_WIDTH && pHitTestInfo->pt.x < rc.left)
        {
          pHitTestInfo->flags |= TVHT_ONITEMBUTTON;
        }
      int delta = STATUS_WIDTH;
      if (pHitTestInfo->pt.x > rc.left && pHitTestInfo->pt.x < rc.left + delta)
        {
          pHitTestInfo->flags |= TVHT_ONITEMICON;
          if ((pHitTestInfo->pt.x - rc.left) / ICON_WIDTH == 0)
            {
              pHitTestInfo->flags |= TVHT_ONITEMICON1;
            }
        }
      rc.left += delta;
      rc.right += delta;
      if (pHitTestInfo->pt.x > rc.left && pHitTestInfo->pt.x < rc.right)
        {
          pHitTestInfo->flags |= TVHT_ONITEMLABEL;
        }
      if (pHitTestInfo->pt.x > rc.right)
        {
          pHitTestInfo->flags |= TVHT_ONITEMRIGHT;
        }
    }
  return lResult;
}*/

/*void CProjectTreeCtrl::
OnTimer (UINT nIDEvent)
{
  KillTimer (1);
  if (m_hPointedItem)
    {
      //EditLabel (m_hPointedItem);
      CProjectItem *pItem = (CProjectItem*) GetItemData (m_hPointedItem);
      if (pItem && (pItem->dwType & PI_FILE))
        {
          ASSERT (g_pProject);
          CString sPath = g_pProject->MakePath (pItem);
          AfxGetMainWnd ()->SendMessage (WM_OPEN_HERE, 0, (LPARAM) (LPCTSTR) sPath);
        }
      else
        {
          if (pItem && !pItem->lstItems.IsEmpty ())
            ExpandItem (m_hPointedItem, TVE_TOGGLE);
        }
      CWnd *pWnd = AfxGetMainWnd ();
      pWnd->SetForegroundWindow ();
      pWnd->SetFocus ();
    }
  CAppTreeCtrl::OnTimer (nIDEvent);
}*/

void CProjectTreeCtrl::
OnSize (UINT nType, int cx, int cy)
{
  CAppTreeCtrl::OnSize (nType, cx, cy);
  if (m_dwLayout & PRJTREEC_HEADER)
    {
      CRect rc;
      m_ctlHeader.GetWindowRect (rc);
      m_ctlHeader.SetWindowPos (NULL, 0, 0, cx, rc.Height (), SWP_NOZORDER | SWP_NOMOVE);
    }
}

#pragma warning ( disable : 4100 )
void CProjectTreeCtrl::
OnRButtonDown (UINT nFlags, CPoint point)
{
  UINT nFlags2;
  m_hPointedItem = HitTest (point, &nFlags2);
  if (m_hPointedItem && (nFlags2 & (TVHT_ONITEMICON | TVHT_ONITEMLABEL)))
    {
      m_bWasSelected = GetItemState (m_hPointedItem, TVIS_SELECTED) != 0;
      if (!m_bWasSelected)
        {
          VERIFY (SetItemState (m_hPointedItem, TVIS_SELECTED, TVIS_SELECTED));
        }
    }
  // CAppTreeCtrl::OnRButtonDown (nFlags, point);
}
#pragma warning ( default : 4100 )

#define ID_SPACE_FIRST ID_REFRESH
#define ID_SPACE_LAST ID_REFRESH

#pragma warning ( disable : 4100 )
void CProjectTreeCtrl::
OnRButtonUp (UINT nFlags, CPoint point)
{
  UINT nFlags2;
  HTREEITEM hItem = HitTest (point, &nFlags2);
  if (m_hPointedItem && (nFlags2 & (TVHT_ONITEMICON | TVHT_ONITEMLABEL)))
    {
      if (hItem == m_hPointedItem)
        {
          CMenu menuItem;
          CMenu *pMenu;
          if (menuItem.LoadMenu (IDM_PRJTREE_ITEM) && (pMenu = menuItem.GetSubMenu (0)) != NULL)
            {
              CProjectItem *pItem = (CProjectItem*) GetItemData (m_hPointedItem);
              if (!(pItem && (pItem->dwType & PI_FILE)))
                pMenu->EnableMenuItem (ID_OPEN, MF_BYCOMMAND|MF_GRAYED);
              if (pItem && (pItem->dwType & PI_FILE))
                {
                  pMenu->EnableMenuItem (ID_NEW_FILE, MF_BYCOMMAND|MF_GRAYED);
                  pMenu->EnableMenuItem (ID_NEW_GROUP, MF_BYCOMMAND|MF_GRAYED);
                  pMenu->EnableMenuItem (ID_ADD_FILE, MF_BYCOMMAND|MF_GRAYED);
                }
              if (!(pItem && !(pItem->dwType & (PI_FILE|PI_SUBITEM))))
                pMenu->EnableMenuItem (ID_RENAME_ITEM, MF_BYCOMMAND|MF_GRAYED);
              if (!GetItemData (m_hPointedItem))
                pMenu->EnableMenuItem (ID_REMOVE_ITEM, MF_BYCOMMAND|MF_GRAYED);
              pMenu->EnableMenuItem (ID_ITEM_PROPERTIES, MF_BYCOMMAND|MF_GRAYED);
              ClientToScreen (&point);
              pMenu->TrackPopupMenu (TPM_LEFTBUTTON|TPM_RIGHTBUTTON/*|TPM_RETURNCMD*/|TPM_LEFTALIGN|TPM_TOPALIGN, point.x, point.y, this);
            }
        }
      if (!m_bWasSelected && m_hPointedItem)
        {
          VERIFY (SetItemState (m_hPointedItem, 0, TVIS_SELECTED));
        }
    }
  else
    {
      CMenu menu;
      if (menu.CreatePopupMenu ())
        {
          DWORD dwResult = 0;
          CWnd *pWnd = this;
          bool bSeparated = false, bContinue = true, bProcess = false, bProperties = false;
          do
            {
              dwResult = pWnd->SendMessage (WM_APPEND_MENU, dwResult, (LPARAM) menu.GetSafeHmenu ());
              if (dwResult & 1)
                {
                  bProcess = true;
                  if (dwResult & 2)
                    {
                      bContinue = false;
                    }
                  if (dwResult & 4)
                    {
                      bSeparated = true;
                    }
                  if (dwResult & 8)
                    {
                      bProperties = true;
                    }
                }
              pWnd = pWnd->GetParent ();
            }
          while (pWnd && bContinue);
          if (bProcess)
            {
              if (bProperties)
                {
                  if (!bSeparated)
                    VERIFY (menu.AppendMenu (MF_SEPARATOR));
                  if (bProperties)
                    {
                      CString sCaption;
                      sCaption.LoadString (IDS_MENU_PROPERTIES);
                      VERIFY (menu.AppendMenu (MF_STRING, ID_PROPERTIES, sCaption));
                    }
                }
              ClientToScreen (&point);
              do
                {
                  DWORD dwReturn = menu.TrackPopupMenu (TPM_LEFTBUTTON|TPM_RIGHTBUTTON|TPM_RETURNCMD|TPM_LEFTALIGN|TPM_TOPALIGN, point.x, point.y, this);
                  bProperties = false;
                  if (dwReturn)
                    {
                      if (dwReturn == ID_PROPERTIES)
                        {
                          SendMessage (WM_COMMAND, dwReturn, (LPARAM) this);
                        }
                      else
                        {
                          dwResult = 0;
                          pWnd = this;
                          bContinue = true;
                          do
                            {
                              dwResult = pWnd->SendMessage (WM_PROCESS_MENU, dwReturn);
                              if (dwResult & 1)
                                {
                                  if (dwResult & 2)
                                    {
                                      bContinue = false;
                                    }
                                  if (dwResult & 4)
                                    {
                                      bProperties = true;
                                    }
                                }
                              pWnd = pWnd->GetParent ();
                            }
                          while (pWnd && bContinue);
                        }
                    }
                }
              while (bProperties);
            }
        }
    }
  // CAppTreeCtrl::OnRButtonUp (nFlags, point);
}
#pragma warning ( default : 4100 )

#pragma warning ( disable : 4100 )
LONG CProjectTreeCtrl::
OnAppendMenu (WPARAM wParam, LPARAM lParam)
{
  CMenu menu;
  CMenu *pMenu;
  if (menu.LoadMenu (IDM_PRJTREE_SPACE) && (pMenu = menu.GetSubMenu (0)) != NULL)
    {
      int nCount = pMenu->GetMenuItemCount ();
      UINT uItem;
      CString sItem;
      DWORD dwResult = 25;
      for (int i = 0; i < nCount; i++)
        {
          uItem = pMenu->GetMenuItemID (i);
          if (uItem)
            {
              dwResult &= ~4;
              pMenu->GetMenuString (i, sItem, MF_BYPOSITION);
              VERIFY (::AppendMenu ((HMENU) lParam, MF_STRING, uItem, sItem));
            }
          else
            {
              dwResult |= 4;
              VERIFY (::AppendMenu ((HMENU) lParam, MF_SEPARATOR, 0, NULL));
            }
        }
      if (!g_pProject)
        ::EnableMenuItem ((HMENU) lParam, ID_REFRESH, MF_BYCOMMAND|MF_GRAYED);
      return dwResult;
  }
  return 0;
}
#pragma warning ( default : 4100 )

#pragma warning ( disable : 4100 )
LONG CProjectTreeCtrl::
OnProcessMenu (WPARAM wParam, LPARAM lParam)
{
  if (wParam >= ID_SPACE_FIRST && wParam <= ID_SPACE_LAST)
    {
      SendMessage (WM_COMMAND, wParam, (LPARAM) this);
      return 1;
    }
  return 0;
}
#pragma warning ( default : 4100 )

#pragma warning ( disable : 4100 )
LONG CProjectTreeCtrl::
OnAppendProps (WPARAM wParam, LPARAM lParam)
{
  /*CDbTreePropPage *pPage = new CDbTreePropPage;
  if (pPage)
    {
      pPage->m_psp.dwFlags &= ~PSP_HASHELP;
      pPage->Init (this);
      ((CPropertySheetPlus*) lParam)->AddPagePlus (pPage);
      return 1;
    }*/
  return 0;
}
#pragma warning ( default : 4100 )

void CProjectTreeCtrl::
OnProperties ()
{
  CPropertySheetPlus sheet (IDS_CAPTION_PROPERTIES, this);
  DWORD dwResult = 0;
  CWnd *pWnd = this;
  bool bContinue = true, bProcess = false;
  do
    {
      dwResult = pWnd->SendMessage (WM_APPEND_PROPS, dwResult, (LPARAM) &sheet);
      if (dwResult & 1)
        {
          bProcess = true;
          if (dwResult & 2)
            {
              bContinue = false;
            }
        }
      pWnd = pWnd->GetParent ();
    }
  while (pWnd && bContinue);
  if (bProcess)
    {
      sheet.m_psh.dwFlags &= ~PSH_HASHELP;
      sheet.m_psh.dwFlags |= PSH_USEHICON;
      sheet.m_psh.hIcon = ::LoadIcon (NULL, MAKEINTRESOURCE (IDI_EDITPAD));
      sheet.m_bDeletePages = true;
      sheet.DoModal ();
    }
}

/////////////////////////////////////////////////////////////////////////////
#pragma warning ( default : 4711 )

void CProjectTreeCtrl::OnOpen()
{
  if (m_hPointedItem)
    {
      if (m_listSelected.IsEmpty () || !m_listSelected.Find (m_hPointedItem))
        {
          CProjectItem *pItem = (CProjectItem*) GetItemData (m_hPointedItem);
          if (pItem && (pItem->dwType & PI_FILE))
            {
              ASSERT (g_pProject);
              CString sPath = g_pProject->MakePath (pItem);
              AfxGetMainWnd ()->SendMessage (WM_OPEN_HERE, 0, (LPARAM) (LPCTSTR) sPath);
            }
        }
      else
        {
          POSITION pos = m_listSelected.GetHeadPosition ();
          ASSERT (pos);
          CProjectItem *pItem;
          do
            {
              pItem = (CProjectItem*) GetItemData (m_listSelected.GetNext (pos));
              if (pItem && (pItem->dwType & PI_FILE))
                {
                  ASSERT (g_pProject);
                  CString sPath = g_pProject->MakePath (pItem);
                  AfxGetMainWnd ()->SendMessage (WM_OPEN_HERE, 0, (LPARAM) (LPCTSTR) sPath);
                }
            }
          while (pos);
        }
      CWnd *pWnd = AfxGetMainWnd ();
      pWnd->SetForegroundWindow ();
      pWnd->SetFocus ();
    }
}

void CProjectTreeCtrl::OnNewFile()
{
  if (m_hPointedItem)
    {
      CProjectItem *pItem = (CProjectItem*) GetItemData (m_hPointedItem);
      CFileDialog oFileDialog (FALSE, _T (""), _T (""), OFN_EXPLORER|OFN_ENABLEHOOK|OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT, _T("Source Files (.c;.cc;.cxx;.cpp;.def;.rc;.rc2)|*.c;*.cc;*.cxx;*.cpp;*.def;*.rc;*.rc2|Header Files (.h;.hpp;.hxx;.inl;.hm)|*.h;*.hpp;*.hxx;*.inl;*.hm|All Files (*.*)|*.*||"), AfxGetMainWnd ());
      static TCHAR szPath[_MAX_PATH];
      oFileDialog.m_ofn.lpstrInitialDir = szPath;
      if (oFileDialog.DoModal () != IDOK)
        return;
      ASSERT (g_pProject);
      CString sFile = oFileDialog.GetPathName (), sPath = GetPath (g_pProject->sPath, TRUE);
      FILE *fp = _tfopen (sFile, _T ("wt"));
      if (fp)
        {
          fclose (fp);
          AfxGetMainWnd ()->SendMessage (WM_OPEN_HERE, 0, (LPARAM) (LPCTSTR) sFile);
          int nLen = sPath.GetLength ();
          if (sPath == sFile.Left (nLen))
            sFile = sFile.Mid (nLen);
          CProjectItem *pNew = new CProjectItem;
          pNew->sPath = sFile;
          pNew->sMidPath = _T ("");
          pNew->dwType = PI_FILE;
          if (pItem)
            pItem->lstItems.AddTail (pNew);
          else
            pNew->dwType |= PI_ROOT;
          g_pProject->lstItems.AddTail (pNew);
          g_pProject->SetModified ();
          CString sName = GetNameExt (sFile);
          int iIcon = GetItemIcon (sFile);
          InsertFile (sName, iIcon, iIcon, (LPARAM) pNew, m_hPointedItem);
          ExpandItem (m_hPointedItem, TVE_EXPAND);
        }
    }
}

void CProjectTreeCtrl::OnNewGroup()
{
  if (m_hPointedItem)
    {
      CProjectItem *pItem = (CProjectItem*) GetItemData (m_hPointedItem);
      ASSERT (g_pProject);
      CProjectItem *pNew = new CProjectItem;
      pNew->sPath = _T ("New Group");
      pNew->sMidPath = _T ("");
      pNew->dwType = PI_GROUP;
      if (pItem)
        pItem->lstItems.AddTail (pNew);
      else
        pNew->dwType |= PI_ROOT;
      g_pProject->lstItems.AddTail (pNew);
      g_pProject->SetModified ();
      InsertGroup (pNew->sPath, 2, 2, (LPARAM) pNew, m_hPointedItem);
      ExpandItem (m_hPointedItem, TVE_EXPAND);
    }
}

void CProjectTreeCtrl::OnAddFile()
{
  if (m_hPointedItem)
    {
      CProjectItem *pItem = (CProjectItem*) GetItemData (m_hPointedItem);
      CEditFileAddDlg dlgFile (TRUE, _T (""), _T (""), OFN_ALLOWMULTISELECT|OFN_FILEMUSTEXIST|OFN_HIDEREADONLY|OFN_EXPLORER|OFN_ENABLEHOOK, _T("Source Files (.c;.cc;.cxx;.cpp;.def;.rc;.rc2)|*.c;*.cc;*.cxx;*.cpp;*.def;*.rc;*.rc2|Header Files (.h;.hpp;.hxx;.inl;.hm)|*.h;*.hpp;*.hxx;*.inl;*.hm|Resource Files (.ico;.bmp;.cur;.gif;.jpg)|*.ico;*.bmp;*.cur;*.gif;*.jpg|Supported Documents (...)|*.lsp;*.bas;*.vb;*.vbs;*.frm;*.dsm;*.bat;*.btm;*.cmd;*.c;*.cc;*.cpp;*.cxx;*.h;*.hpp;*.hxx;*.hm;*.inl;*.rh;*.tlh;*.tli;*.xs;*.dcl;*.dcc;*.f;*.f90;*.f9p;*.fpp;*.html;*.htm;*.shtml;*.ihtml;*.ssi;*.rul;*.java;*.jav;*.pas;*.pl;*.txt;*.doc;*.diz;*.py;*.rc;*.dlg;*.rex;*.rexx;*.sgml;*.sh;*.conf;*.scm;*.sql;*.tcl;*.text;*.sty;*.cls;*.clo;*.ltx;*.fd;*.dtx|All Files (*.*)|*.*||"), AfxGetMainWnd ());
      static TCHAR szFileListBuffer[8192];
      static TCHAR szPath[_MAX_PATH];
      dlgFile.m_ofn.lpstrInitialDir = szPath;
      *szFileListBuffer = _T ('\0');
      dlgFile.m_ofn.lpstrFile = szFileListBuffer;
      dlgFile.m_ofn.nMaxFile = countof (szFileListBuffer);
      if (dlgFile.DoModal () == IDOK)
        {
          ASSERT (g_pProject);
          POSITION pos = dlgFile.GetStartPosition ();
          CString sFile, sPath = GetPath (g_pProject->sPath, TRUE);
          int nLen = sPath.GetLength ();
          while (pos)
            {
              sFile = dlgFile.GetNextPathName (pos);
              if (sPath == sFile.Left (nLen))
                sFile = sFile.Mid (nLen);
              CProjectItem *pNew = new CProjectItem;
              pNew->sPath = sFile;
              pNew->sMidPath = _T ("");
              pNew->dwType = PI_FILE;
              if (pItem)
                pItem->lstItems.AddTail (pNew);
              else
                pNew->dwType |= PI_ROOT;
              g_pProject->lstItems.AddTail (pNew);
              g_pProject->SetModified ();
              CString sName = GetNameExt (sFile);
              int iIcon = GetItemIcon (sFile);
              InsertFile (sName, iIcon, iIcon, (LPARAM) pNew, m_hPointedItem);
            }
          ExpandItem (m_hPointedItem, TVE_EXPAND);
        }
    }
}

void CProjectTreeCtrl::OnRenameItem()
{
  if (m_hPointedItem)
    EditLabel (m_hPointedItem);
}

void CProjectTreeCtrl::OnRemoveItem()
{
  if (m_hPointedItem)
    {
      if (m_listSelected.IsEmpty () || !m_listSelected.Find (m_hPointedItem))
        {
          CProjectItem *pItem = (CProjectItem*) GetItemData (m_hPointedItem);
        	
          if (pItem)
            {
              ASSERT (g_pProject);
              POSITION pos = g_pProject->lstItems.Find (pItem);
              if (pos)
                {
                  g_pProject->RemoveItems (pos);
                  HTREEITEM hItem = GetParentItem (m_hPointedItem);
                  if (hItem)
                    {
                      CProjectItem *pParent = (CProjectItem*) GetItemData (hItem);
                      if (pParent)
                        {
                          pos = pParent->lstItems.Find (pItem);
                          if (pos)
                            pParent->lstItems.RemoveAt (pos);
                        }
                      if (GetItemCount (hItem) == 1)
                        ExpandItem (hItem, TVE_COLLAPSE);
                    }
                  g_pProject->SetModified ();
                }
              DeleteItem (m_hPointedItem);
            }
        }
      else
        {
          POSITION pos = m_listSelected.GetHeadPosition (), oldpos;
          ASSERT (pos);
          CProjectItem *pItem;
          HTREEITEM hItem;
          do
            {
              oldpos = pos;
              hItem = m_listSelected.GetNext (pos);
              pItem = (CProjectItem*) GetItemData (hItem);
              if (pItem && (pItem->dwType & PI_FILE))
                {
                  ASSERT (g_pProject);
                  POSITION pos2 = g_pProject->lstItems.Find (pItem);
                  if (pos2)
                    {
                      HTREEITEM hParent = GetParentItem (hItem);
                      if (hParent)
                        {
                          CProjectItem *pParent = (CProjectItem*) GetItemData (hParent);
                          if (pParent)
                            {
                              POSITION pos3 = pParent->lstItems.Find (pItem);
                              if (pos3)
                                pParent->lstItems.RemoveAt (pos3);
                            }
                          if (GetItemCount (hParent) == 1)
                            ExpandItem (hParent, TVE_COLLAPSE);
                        }
                      g_pProject->RemoveItems (pos2);
                    }
                  DeleteItem (hItem);
                  m_listSelected.RemoveAt (oldpos);
                  pos = m_listSelected.GetHeadPosition ();
                }
            }
          while (pos);
          pos = m_listSelected.GetHeadPosition ();
          while (pos)
            {
              hItem = m_listSelected.GetNext (pos);
              pItem = (CProjectItem*) GetItemData (hItem);
              if (pItem && (pItem->dwType & PI_GROUP))
                {
                  ASSERT (g_pProject);
                  POSITION pos2 = g_pProject->lstItems.Find (pItem);
                  if (pos2)
                    {
                      HTREEITEM hParent = GetParentItem (hItem);
                      if (hParent)
                        {
                          CProjectItem *pParent = (CProjectItem*) GetItemData (hParent);
                          if (pParent)
                            {
                              POSITION pos3 = pParent->lstItems.Find (pItem);
                              if (pos3)
                                pParent->lstItems.RemoveAt (pos3);
                            }
                          if (GetItemCount (hParent) == 1)
                            ExpandItem (hParent, TVE_COLLAPSE);
                        }
                      g_pProject->RemoveItems (pos2);
                    }
                  DeleteItem (hItem);
                }
            }
          m_listSelected.RemoveAll ();
          g_pProject->SetModified ();
        }
      m_hPointedItem = NULL;
      CWnd *pWnd = AfxGetMainWnd ();
      pWnd->SetForegroundWindow ();
      pWnd->SetFocus ();
    }
}

void CProjectTreeCtrl::OnRefresh()
{
  Populate ();
}

void CProjectTreeCtrl::OnItemProperties()
{
	// TODO: Add your command handler code here
	
}
