///////////////////////////////////////////////////////////////////////////
//  File:    mainfrm.cpp
//  Version: 1.1.0.4
//  Updated: 19-Jul-1998
//
//  Main frame core
//
//  Copyright:  Ferdinand Prantl, portions by Stcherbatchenko Andrei
//  E-mail:     prantl@ff.cuni.cz
//
//  You are free to use or modify this code to the following restrictions:
//  - Acknowledge me somewhere in your about box, simple "Parts of code by.."
//  will be enough. If you can't (or don't want to), contact me personally.
//  - LEAVE THIS HEADER INTACT
////////////////////////////////////////////////////////////////////////////

// mainfrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "editpad.h"

#include "mainfrm.h"
#include "editcmd.h"
#include "registry.h"
#include "editreg.h"
#include "ccrystaltextbuffer.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC (CMainFrame, CBCGMDIFrameWnd)

#define CONTROL_BARS _T("Control Bars")

const int iMaxUserToolbars = 10;
const UINT uiFirstUserToolBarId	= AFX_IDW_CONTROLBAR_FIRST + 40;
const UINT uiLastUserToolBarId	= uiFirstUserToolBarId + iMaxUserToolbars - 1;

BEGIN_MESSAGE_MAP (CMainFrame, CBCGMDIFrameWnd)
//{{AFX_MSG_MAP(CMainFrame)
ON_WM_CREATE ()
ON_WM_CLOSE()
ON_WM_SETFOCUS()
ON_COMMAND(ID_VIEW_FULL_SCREEN, OnViewFullScreen)
ON_UPDATE_COMMAND_UI(ID_VIEW_FULL_SCREEN, OnUpdateViewFullScreen)
ON_COMMAND(ID_FILE_CLOSE_ALL, OnFileCloseAll)
ON_COMMAND(ID_WINDOW_CLOSE, OnCloseWindow)
ON_COMMAND(ID_WINDOW_NEXT, OnNextWindow)
ON_COMMAND(ID_WINDOW_PREV, OnPrevWindow)
ON_UPDATE_COMMAND_UI(ID_WINDOW_CLOSE, OnUpdateCloseWindow)
ON_UPDATE_COMMAND_UI(ID_WINDOW_NEXT, OnUpdateNextWindow)
ON_UPDATE_COMMAND_UI(ID_WINDOW_PREV, OnUpdatePrevWindow)
ON_COMMAND_EX_RANGE(ID_ENCODING_FIRST, ID_ENCODING_LAST, OnEncoding)
ON_UPDATE_COMMAND_UI(ID_ENCODING_FIRST, OnUpdateEncoding)
//}}AFX_MSG_MAP
// Global help commands
ON_COMMAND (ID_HELP_FINDER, CBCGMDIFrameWnd::OnHelpFinder)
ON_COMMAND (ID_HELP, CBCGMDIFrameWnd::OnHelp)
ON_COMMAND (ID_CONTEXT_HELP, CBCGMDIFrameWnd::OnContextHelp)
ON_COMMAND (ID_DEFAULT_HELP, CBCGMDIFrameWnd::OnHelpFinder)
ON_COMMAND (ID_VIEW_CUSTOMIZE, OnViewCustomize)
ON_REGISTERED_MESSAGE (BCGM_RESETTOOLBAR, OnToolbarReset)
ON_REGISTERED_MESSAGE (BCGM_TOOLBARMENU, OnToolbarContextMenu)
ON_COMMAND_EX_RANGE(ID_VIEW_USER_TOOLBAR1, ID_VIEW_USER_TOOLBAR1 + iMaxUserToolbars - 1, OnToolsViewUserToolbar)
ON_UPDATE_COMMAND_UI_RANGE(ID_VIEW_USER_TOOLBAR1, ID_VIEW_USER_TOOLBAR1 + iMaxUserToolbars - 1, OnUpdateToolsViewUserToolbar)
ON_REGISTERED_MESSAGE (BCGM_CUSTOMIZEHELP, OnHelpCustomizeToolbars)
END_MESSAGE_MAP ()

static UINT indicators[] =
  {
    ID_SEPARATOR,            // status line indicator
    // ID_EDIT_INDICATOR_POSITION,
    // ID_EDIT_INDICATOR_COL,
    // ID_INDICATOR_OVR,
    // ID_EDIT_INDICATOR_READ,
    // ID_EDIT_INDICATOR_CRLF,
    ID_INDICATOR_CAPS,
    ID_INDICATOR_NUM,
    ID_INDICATOR_SCRL,
    ID_INDICATOR_TIME
  };

static UINT indicators1[] =
  {
    ID_SEPARATOR,            // status line indicator
    ID_EDIT_INDICATOR_POSITION,
    ID_INDICATOR_OVR,
    ID_EDIT_INDICATOR_READ,
    ID_EDIT_INDICATOR_CRLF,
    ID_INDICATOR_CAPS,
    ID_INDICATOR_NUM,
    ID_INDICATOR_SCRL,
    ID_INDICATOR_TIME
  };

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame ()
{
	m_bFullScreen = false;
}

CMainFrame:: ~ CMainFrame ()
{
}

void CMainFrame::FullScreenOn ()
{
	// available only if there is an active doc
	CMDIChildWnd *pChild = MDIGetActive();
	if (!pChild)
    return;
	// m_bToolBarWasVisible = m_wndToolBar.IsWindowVisible () != FALSE;
  // if (m_bToolBarWasVisible)
	//   m_wndToolBar.ShowWindow (SW_HIDE);
	m_bStatusBarWasVisible = m_wndStatusBar.IsWindowVisible () != FALSE;
  if (m_bStatusBarWasVisible)
	  m_wndStatusBar.ShowWindow (SW_HIDE);
	// first create the new toolbar
	// this will contain the full-screen off button
	m_pwndFullScreenBar = new CToolBar;
	m_pwndFullScreenBar->Create (this);
	m_pwndFullScreenBar->LoadToolBar (IDR_FULLSCREEN);
	m_pwndFullScreenBar->SetBarStyle (m_pwndFullScreenBar->GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	// to look better:
	m_pwndFullScreenBar->ModifyStyle (0, TBSTYLE_FLAT);
	m_pwndFullScreenBar->EnableDocking (0);
	// place the full-screen off button somewhere:
	CPoint pt(600, 440);
	FloatControlBar (m_pwndFullScreenBar, pt);
	// remove the caption of the mainWnd:
	LONG style =:: GetWindowLong (m_hWnd, GWL_STYLE);
	m_bMax = style & WS_MAXIMIZE ? true : false;
	if (m_bMax)
    ShowWindow (SW_RESTORE);
	// now save the old positions of the main and child windows
	GetWindowRect (&m_mainRect);
	style &= ~(WS_CAPTION|WS_MAXIMIZE);
	::SetWindowLong (m_hWnd, GWL_STYLE, style);
	int screenx = GetSystemMetrics (SM_CXSCREEN);
	int screeny = GetSystemMetrics (SM_CYSCREEN);
	// resize:
	SetWindowPos (NULL, -4, -4, screenx + 8, screeny + 8, SWP_NOZORDER);
	style = ::GetWindowLong (pChild->m_hWnd, GWL_STYLE);
	m_bChildMax = style & WS_MAXIMIZE ? true : false;
	// note here: m_bMainMax is not needed since m_hWnd only
	// changed its caption...
	// and maximize the child window
	// it will remove its caption, too.
	pChild->ShowWindow (SW_SHOWMAXIMIZED);
	RecalcLayout ();
	m_bFullScreen = true;
}

void CMainFrame::FullScreenOff ()
{
	// You can use SaveBarState() in OnClose(),
	// so remove the newly added toolbar entirely
	// in order SaveBarState() not
	// to save its state. That is why I used dynamic
	// allocation
	delete m_pwndFullScreenBar;
	LONG style = ::GetWindowLong (m_hWnd, GWL_STYLE);
	style |= WS_CAPTION;
	::SetWindowLong (m_hWnd, GWL_STYLE, style);
	// if (m_bToolBarWasVisible)
	//	m_wndToolBar.ShowWindow (SW_SHOW);
	if (m_bStatusBarWasVisible)
		m_wndStatusBar.ShowWindow (SW_SHOW);
	MoveWindow (&m_mainRect);
	RecalcLayout ();
	if (m_bMax)
    ShowWindow (SW_SHOWMAXIMIZED);
	CMDIChildWnd *pChild = MDIGetActive ();
	// pchild can be NULL if the USER closed all the
	// childs during Full Screen Mode:
	if(pChild)
    {
		  if(m_bChildMax)
			  MDIMaximize (pChild);
		  else
        MDIRestore (pChild);
	  }
	m_bFullScreen = false;
}

int CMainFrame::MDIGetCount ()
{
  ASSERT (::IsWindow (m_hWnd));
  int nCount = 0;
  CWnd * pWnd = GetWindow (GW_CHILD);
  ASSERT (pWnd);
  pWnd = pWnd->GetWindow (GW_CHILD);
  while (pWnd)
    {
      nCount++;
      pWnd = pWnd->GetWindow (GW_HWNDNEXT);
    }
  return nCount;
}

BOOL CMainFrame::
PreCreateWindow (CREATESTRUCT & cs)
{
  CReg reg;
  if (reg.Open (HKEY_CURRENT_USER, REG_EDITPAD_MULTI, KEY_READ))
    {
      // reg.LoadNumber (_T ("ShowFlags"), (DWORD*) &status.flags);
      reg.LoadNumber (_T ("ShowCmd"), (DWORD*) &theApp.m_nCmdShow);

      if (reg.LoadNumber (_T ("Left"), (DWORD*) &cs.x) &&
        reg.LoadNumber (_T ("Right"), (DWORD*) &cs.cx))
        {
          cs.cx -= cs.x;
        }
      if (reg.LoadNumber (_T ("Top"), (DWORD*) &cs.y) &&
        reg.LoadNumber (_T ("Bottom"), (DWORD*) &cs.cy))
        {
          cs.cy -= cs.y;
        }
    }

  if (!CBCGMDIFrameWnd::PreCreateWindow (cs))
    return FALSE;

  return TRUE;
}

void CMainFrame::
ChooseStatusBar (BOOL bActivate)
{
  if (m_bActivated != bActivate)
    {
      if (bActivate)
        m_wndStatusBar.SetIndicators (indicators1, sizeof (indicators1) / sizeof (UINT));
      else
        m_wndStatusBar.SetIndicators (indicators, sizeof (indicators) / sizeof (UINT));
      m_bActivated = bActivate;
    }
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::
AssertValid ()
const
{
  CBCGMDIFrameWnd::AssertValid ();
}

void CMainFrame::Dump (CDumpContext & dc)
const
{
  CBCGMDIFrameWnd::Dump (dc);
}

#endif                          //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

int CMainFrame::
OnCreate (LPCREATESTRUCT lpCreateStruct)
{
  if (CBCGMDIFrameWnd::OnCreate (lpCreateStruct) == -1)
    return -1;

  //--------------------------
  // Load toolbar user images:
  //--------------------------
  if (!m_UserImages.Load (_T(".\\usrimgs.bmp")))
  {
    TRACE(_T("Failed to load user images\n"));
    ASSERT (FALSE);
  }
  else
  {
    CBCGToolBar::SetUserImages (&m_UserImages);
  }

  CBCGToolBar::EnableQuickCustomization ();

  CList<UINT, UINT>	lstBasicCoomads;

  lstBasicCoomads.AddTail (ID_FILE_NEW);
  lstBasicCoomads.AddTail (ID_FILE_OPEN);
  lstBasicCoomads.AddTail (ID_FILE_CLOSE);
  lstBasicCoomads.AddTail (ID_FILE_SAVE);
  lstBasicCoomads.AddTail (ID_FILE_PRINT);
  lstBasicCoomads.AddTail (ID_APP_EXIT);
  lstBasicCoomads.AddTail (ID_EDIT_UNDO);
  lstBasicCoomads.AddTail (ID_EDIT_CUT);
  lstBasicCoomads.AddTail (ID_EDIT_COPY);
  lstBasicCoomads.AddTail (ID_EDIT_PASTE);
  lstBasicCoomads.AddTail (ID_EDIT_FIND);
  lstBasicCoomads.AddTail (ID_EDIT_REPLACE);
  lstBasicCoomads.AddTail (ID_VIEW_TOOLBAR);
  lstBasicCoomads.AddTail (ID_VIEW_TOGGLE_SRC_HDR);
  lstBasicCoomads.AddTail (ID_VIEW_CUSTOMIZE);
  lstBasicCoomads.AddTail (ID_VIEW_OPTIONS);
  lstBasicCoomads.AddTail (ID_VIEW_OPTIONS_GENERAL);
  lstBasicCoomads.AddTail (ID_WINDOW_NEW);
  lstBasicCoomads.AddTail (ID_WINDOW_NEXT);
  lstBasicCoomads.AddTail (ID_WINDOW_TILE_HORZ);
  lstBasicCoomads.AddTail (ID_WINDOW_CASCADE);
  lstBasicCoomads.AddTail (ID_WINDOW_CLOSE);
  lstBasicCoomads.AddTail (ID_FILE_CLOSE_ALL);
  lstBasicCoomads.AddTail (ID_APP_ABOUT);
  lstBasicCoomads.AddTail (ID_HELP_FINDER);

  CBCGToolBar::SetBasicCommands (lstBasicCoomads);
  if (!m_wndMenuBar.Create (this))
  {
    TRACE0 ("Failed to create menubar\n");
    return -1;      // fail to create
  }

  m_wndMenuBar.SetBarStyle (m_wndMenuBar.GetBarStyle() | CBRS_SIZE_DYNAMIC);
  if (!m_wndToolBar.CreateEx (this, TBSTYLE_FLAT, WS_CHILD|WS_VISIBLE|CBRS_TOP|CBRS_GRIPPER|CBRS_TOOLTIPS|CBRS_FLYBY|CBRS_SIZE_DYNAMIC) ||
    !m_wndToolBar.LoadToolBar (IDR_MAINFRAME))
  {
    TRACE0("Failed to create toolbar\n");
    return -1;      // fail to create
  }

  if (!m_wndStatusBar.Create (this) ||
        !m_wndStatusBar.SetIndicators (indicators,
                                       sizeof (indicators) / sizeof (UINT)))
    {
      TRACE0 ("Failed to create status bar\n");
      return -1;                // fail to create

    }

  m_bActivated = FALSE;

  CString strMainToolbarTitle;
  strMainToolbarTitle.LoadString (IDS_MAIN_TOOLBAR);
  m_wndToolBar.SetWindowText (strMainToolbarTitle);
  m_wndMenuBar.EnableDocking (CBRS_ALIGN_ANY);
  m_wndToolBar.EnableDocking (CBRS_ALIGN_ANY);
  EnableDocking (CBRS_ALIGN_ANY);
  m_wndToolBar.EnableCustomizeButton (TRUE, ID_VIEW_CUSTOMIZE, _T("Customize..."));
  DockControlBar (&m_wndMenuBar);
  DockControlBar (&m_wndToolBar);

  CString strControlBarRegEntry = REGISTRY_ROOT;
  strControlBarRegEntry += CONTROL_BARS;

  //--------------------------------------
  // Load general toolbar/menu parameters:
  //--------------------------------------
  CBCGToolBar::LoadParameters (strControlBarRegEntry);

  //----------------------------------------
  // Allow user-defined toolbars operations:
  //----------------------------------------
  InitUserToobars (strControlBarRegEntry, uiFirstUserToolBarId, uiLastUserToolBarId);

  //------------------------
  // Load standard toolbars:
  //------------------------
  m_wndToolBar.LoadState (strControlBarRegEntry);
  m_wndMenuBar.LoadState (strControlBarRegEntry);

  //-------------------------------------
  // Load control bar states and context:
  //-------------------------------------
  CDockState dockState;
  dockState.LoadState (CONTROL_BARS);

  if (IsDockStateValid (dockState))
  {
    SetDockState (dockState);
  }

  return 0;
}

void CMainFrame::OnClose()
{
  theApp.SaveLastDocs ();
  if (m_bFullScreen)
    FullScreenOff ();
  WINDOWPLACEMENT status;
  status.length = sizeof (WINDOWPLACEMENT);
  GetWindowPlacement (&status);

  CReg reg;
  if (reg.Create (HKEY_CURRENT_USER, REG_EDITPAD_MULTI, KEY_WRITE))
    {
      VERIFY (reg.SaveNumber (_T ("ShowFlags"), status.flags));
      VERIFY (reg.SaveNumber (_T ("ShowCmd"), status.showCmd));

      VERIFY (reg.SaveNumber (_T ("Left"), status.rcNormalPosition.left));
      VERIFY (reg.SaveNumber (_T ("Top"), status.rcNormalPosition.top));
      VERIFY (reg.SaveNumber (_T ("Right"), status.rcNormalPosition.right));
      VERIFY (reg.SaveNumber (_T ("Bottom"), status.rcNormalPosition.bottom));
    }
  // SaveBarState(theApp.m_pszProfileName);
  CString strControlBarRegEntry = REGISTRY_ROOT;
  strControlBarRegEntry += CONTROL_BARS;

  //--------------------------------------
  // Save general toolbar/menu parameters:
  //--------------------------------------
  CBCGToolBar::SaveParameters (strControlBarRegEntry);

  CDockState dockState;
  GetDockState (dockState);
  dockState.SaveState (CONTROL_BARS);

  //------------------------
  // Save standard toolbars:
  //------------------------
  m_wndToolBar.SaveState (strControlBarRegEntry);
  m_wndMenuBar.SaveState (strControlBarRegEntry);

  //----------------------------
  // Save user defined toolbars:
  //----------------------------
  SaveUserToolbars ();

  //------------------
  // Save user images:
  //------------------
  m_UserImages.Save ();

  theApp.SaveCustomState ();

  CBCGMDIFrameWnd::OnClose();
}

void CMainFrame::OnViewCustomize()
{
  //------------------------------------
  // Create a customize toolbars dialog:
  //------------------------------------
  CBCGToolbarCustomize* pDlgCust = new CBCGToolbarCustomize (this,  TRUE /* Automatic menus scaning */);

  pDlgCust->EnableUserDefinedToolbars ();
  pDlgCust->Create ();
}

void CMainFrame::OnViewFullScreen()
{
	if (m_bFullScreen)
		FullScreenOff ();
	else
    FullScreenOn ();
}

void CMainFrame::OnUpdateViewFullScreen(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck (m_bFullScreen);	
}

void CMainFrame::OnFileCloseAll()
{
  if (theApp.SaveAllModified ())
    theApp.CloseAllDocuments (FALSE);
}

void CMainFrame::OnCloseWindow()
{
  MDIGetActive ()->DestroyWindow ();
}

void CMainFrame::OnUpdateCloseWindow(CCmdUI* pCmdUI)
{
  pCmdUI->Enable (MDIGetActive () != NULL);
}

void CMainFrame::OnNextWindow()
{
  MDINext ();
}

void CMainFrame::OnUpdateNextWindow(CCmdUI* pCmdUI)
{
  pCmdUI->Enable (MDIGetCount () > 1);
}

void CMainFrame::OnPrevWindow()
{
  MDIPrev ();
}

void CMainFrame::OnUpdatePrevWindow(CCmdUI* pCmdUI)
{
  pCmdUI->Enable (MDIGetCount () > 1);
}

//Alarm flashing until the window became focus
void CMainFrame::FlashUntilFocus ()
  {
	  m_bGotFocus = FALSE;

	  BOOL bInvert = FALSE;
	  while (!m_bGotFocus)
	    {		
		    FlashWindow (bInvert);
		    bInvert= !bInvert;
		    Sleep (500);
		    if (GetFocus() == this)
			    break;
	    }		
	  FlashWindow(FALSE);
  }

void CMainFrame::OnSetFocus (CWnd* pOldWnd)
{
	CBCGMDIFrameWnd::OnSetFocus (pOldWnd);	
	m_bGotFocus = TRUE;	
}

LRESULT CMainFrame::OnToolbarContextMenu(WPARAM,LPARAM lp)
{
  /*CPoint point (LOWORD (lp), HIWORD(lp));

  CMenu menu;
  VERIFY(menu.LoadMenu (IDR_POPUP_TOOLBAR));

  CMenu* pPopup = menu.GetSubMenu(0);
  ASSERT(pPopup != NULL);


  CBCGPopupMenu* pPopupMenu = new CBCGPopupMenu;
  pPopupMenu->Create (this, point.x, point.y, pPopup->Detach ());*/

  CPoint point (LOWORD (lp), HIWORD(lp));

  CMenu menu;
  VERIFY(menu.LoadMenu (IDR_POPUP_TOOLBAR));

  CMenu* pPopup = menu.GetSubMenu(0);
  ASSERT(pPopup != NULL);

  //---------------------------------------------------------------
  // Replace toolbar dummy items to the user-defined toolbar names:
  //---------------------------------------------------------------
  for (int i = 0; i < (int) pPopup->GetMenuItemCount ();)
    {
      UINT uiCmd = pPopup->GetMenuItemID (i);

      if (uiCmd >= ID_VIEW_USER_TOOLBAR1 && uiCmd < ID_VIEW_USER_TOOLBAR1 + iMaxUserToolbars)
        {
          //-------------------------------------------------------------------
          // "User toolbar" item. First check that toolbar number 'x' is exist:
          //-------------------------------------------------------------------
          CBCGToolBar* pToolBar = GetUserBarByIndex (uiCmd - ID_VIEW_USER_TOOLBAR1);
          if (pToolBar != NULL)
            {
              //-----------------------------------------------------------
              // Modify the current menu item text to the toolbar title and
              // move next:
              //-----------------------------------------------------------
              CString strToolbarName;
              pToolBar->GetWindowText (strToolbarName);

              pPopup->ModifyMenu (i ++, MF_BYPOSITION | MF_STRING, uiCmd, strToolbarName);
            }
          else
            {
              pPopup->DeleteMenu (i, MF_BYPOSITION);
            }
        }
      else	// Not "user toolbar" item, move next
        {
          i ++;
        }
    }

  CBCGPopupMenu* pPopupMenu = new CBCGPopupMenu;
  pPopupMenu->Create (this, point.x, point.y, pPopup->Detach ());

  return 0;
}

void CMainFrame::OnToolsViewUserToolbar (UINT uiId)
{
  CBCGToolBar* pUserToolBar = GetUserBarByIndex (uiId - ID_VIEW_USER_TOOLBAR1);
  if (pUserToolBar == NULL)
    {
      ASSERT (FALSE);
      return;
    }

  ShowControlBar (pUserToolBar, !(pUserToolBar->GetStyle () & WS_VISIBLE), FALSE);
  RecalcLayout ();
}

void CMainFrame::OnUpdateToolsViewUserToolbar (CCmdUI* pCmdUI)
{
  CBCGToolBar* pUserToolBar = GetUserBarByIndex (pCmdUI->m_nID - ID_VIEW_USER_TOOLBAR1);
  if (pUserToolBar == NULL)
    {
      pCmdUI->Enable (FALSE);
      return;
    }

  pCmdUI->Enable ();
  pCmdUI->SetCheck (pUserToolBar->GetStyle () & WS_VISIBLE);
}

afx_msg LRESULT CMainFrame::OnToolbarReset(WPARAM /*wp*/,LPARAM)
{
  // TODO: reset toolbar with id = (UINT) wp to its initial state:
  //
  // UINT uiToolBarId = (UINT) wp;
  // if (uiToolBarId == IDR_MAINFRAME)
  // {
  //		do something with m_wndToolBar
  // }

  return 0;
}

LRESULT CMainFrame::OnHelpCustomizeToolbars(WPARAM wp, LPARAM lp)
{
  int iPageNum = (int) wp;

  CBCGToolbarCustomize* pDlg = (CBCGToolbarCustomize*) lp;
  ASSERT_VALID (pDlg);

  // TODO: show help about page number iPageNum


  return 0;
}

extern EDITPADC_CLASS LPCTSTR pszCodeNames[];
extern EDITPADC_CLASS int nCodeNames;

void CMainFrame::OnEncoding(UINT nID)
{
  ASSERT_VALID (this);
  ASSERT (nID >= ID_ENCODING_1);
  ASSERT (nID < (UINT) (ID_ENCODING_1 + nCodeNames + 1));
  CCrystalTextBuffer::m_nDefaultEncoding = nID - ID_ENCODING_1 - 1;
}

void CMainFrame::OnUpdateEncoding(CCmdUI* pCmdUI)
{
  CMenu *pMenu = pCmdUI->m_pMenu ? pCmdUI->m_pMenu->GetSubMenu (0) : NULL;

  if (!pMenu)
    return;

  for (int iEnc = 0; iEnc < 16; iEnc++)
    pMenu->DeleteMenu (pCmdUI->m_nID + iEnc, MF_BYCOMMAND);

  TCHAR buf[40];
  LPCTSTR *pszCodeName = pszCodeNames;
  iEnc = 0;
  _stprintf (buf, _T("&%d %s"), iEnc++ % 10, _T ("No recoding"));
  pMenu->InsertMenu (pCmdUI->m_nIndex++, MF_STRING|MF_BYPOSITION|((DWORD) CCrystalTextBuffer::m_nDefaultEncoding == pCmdUI->m_nID - ID_ENCODING_1 - 1 ? MF_CHECKED : 0), pCmdUI->m_nID, buf);
  pCmdUI->m_nID++;
  do
    {
      _stprintf (buf, _T("&%d %s"), iEnc++ % 10, *pszCodeName++);
      pMenu->InsertMenu (pCmdUI->m_nIndex++, MF_STRING|MF_BYPOSITION|((DWORD) CCrystalTextBuffer::m_nDefaultEncoding == pCmdUI->m_nID - ID_ENCODING_1 - 1 ? MF_CHECKED : 0), pCmdUI->m_nID, buf);
      pCmdUI->m_nID++;
    }
  while (*pszCodeName);

  pCmdUI->m_nIndex--;
  pCmdUI->m_nIndexMax = pMenu->GetMenuItemCount();

  pCmdUI->m_bEnableChanged = TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame2

IMPLEMENT_DYNAMIC (CMainFrame2, CMDIFrameWnd)

BEGIN_MESSAGE_MAP (CMainFrame2, CMDIFrameWnd)
//{{AFX_MSG_MAP(CMainFrame2)
ON_WM_CREATE ()
ON_WM_CLOSE()
ON_WM_SETFOCUS()
ON_COMMAND(ID_VIEW_FULL_SCREEN, OnViewFullScreen)
ON_UPDATE_COMMAND_UI(ID_VIEW_FULL_SCREEN, OnUpdateViewFullScreen)
ON_COMMAND(ID_FILE_CLOSE_ALL, OnFileCloseAll)
ON_COMMAND(ID_WINDOW_CLOSE, OnCloseWindow)
ON_COMMAND(ID_WINDOW_NEXT, OnNextWindow)
ON_COMMAND(ID_WINDOW_PREV, OnPrevWindow)
ON_UPDATE_COMMAND_UI(ID_WINDOW_CLOSE, OnUpdateCloseWindow)
ON_UPDATE_COMMAND_UI(ID_WINDOW_NEXT, OnUpdateNextWindow)
ON_UPDATE_COMMAND_UI(ID_WINDOW_PREV, OnUpdatePrevWindow)
ON_COMMAND_EX_RANGE(ID_ENCODING_FIRST, ID_ENCODING_LAST, OnEncoding)
ON_UPDATE_COMMAND_UI(ID_ENCODING_FIRST, OnUpdateEncoding)
//}}AFX_MSG_MAP
// Global help commands
ON_COMMAND (ID_HELP_FINDER, CMDIFrameWnd::OnHelpFinder)
ON_COMMAND (ID_HELP, CMDIFrameWnd::OnHelp)
ON_COMMAND (ID_CONTEXT_HELP, CMDIFrameWnd::OnContextHelp)
ON_COMMAND (ID_DEFAULT_HELP, CMDIFrameWnd::OnHelpFinder)
END_MESSAGE_MAP ()

/////////////////////////////////////////////////////////////////////////////
// CMainFrame2 construction/destruction

CMainFrame2::CMainFrame2 ()
{
	m_bFullScreen = false;
}

CMainFrame2:: ~ CMainFrame2 ()
{
}

void CMainFrame2::FullScreenOn ()
{
	// available only if there is an active doc
	CMDIChildWnd *pChild = MDIGetActive();
	if (!pChild)
    return;
	// m_bToolBarWasVisible = m_wndToolBar.IsWindowVisible () != FALSE;
  // if (m_bToolBarWasVisible)
	//   m_wndToolBar.ShowWindow (SW_HIDE);
	m_bStatusBarWasVisible = m_wndStatusBar.IsWindowVisible () != FALSE;
  if (m_bStatusBarWasVisible)
	  m_wndStatusBar.ShowWindow (SW_HIDE);
	// first create the new toolbar
	// this will contain the full-screen off button
	m_pwndFullScreenBar = new CToolBar;
	m_pwndFullScreenBar->Create (this);
	m_pwndFullScreenBar->LoadToolBar (IDR_FULLSCREEN);
	m_pwndFullScreenBar->SetBarStyle (m_pwndFullScreenBar->GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	// to look better:
	m_pwndFullScreenBar->ModifyStyle (0, TBSTYLE_FLAT);
	m_pwndFullScreenBar->EnableDocking (0);
	// place the full-screen off button somewhere:
	CPoint pt(600, 440);
	FloatControlBar (m_pwndFullScreenBar, pt);
	// remove the caption of the mainWnd:
	LONG style =:: GetWindowLong (m_hWnd, GWL_STYLE);
	m_bMax = style & WS_MAXIMIZE ? true : false;
	if (m_bMax)
    ShowWindow (SW_RESTORE);
	// now save the old positions of the main and child windows
	GetWindowRect (&m_mainRect);
	style &= ~(WS_CAPTION|WS_MAXIMIZE);
	::SetWindowLong (m_hWnd, GWL_STYLE, style);
	int screenx = GetSystemMetrics (SM_CXSCREEN);
	int screeny = GetSystemMetrics (SM_CYSCREEN);
	// resize:
	SetWindowPos (NULL, -4, -4, screenx + 8, screeny + 8, SWP_NOZORDER);
	style = ::GetWindowLong (pChild->m_hWnd, GWL_STYLE);
	m_bChildMax = style & WS_MAXIMIZE ? true : false;
	// note here: m_bMainMax is not needed since m_hWnd only
	// changed its caption...
	// and maximize the child window
	// it will remove its caption, too.
	pChild->ShowWindow (SW_SHOWMAXIMIZED);
	RecalcLayout ();
	m_bFullScreen = true;
}

void CMainFrame2::FullScreenOff ()
{
	// You can use SaveBarState() in OnClose(),
	// so remove the newly added toolbar entirely
	// in order SaveBarState() not
	// to save its state. That is why I used dynamic
	// allocation
	delete m_pwndFullScreenBar;
	LONG style = ::GetWindowLong (m_hWnd, GWL_STYLE);
	style |= WS_CAPTION;
	::SetWindowLong (m_hWnd, GWL_STYLE, style);
	// if (m_bToolBarWasVisible)
	//	m_wndToolBar.ShowWindow (SW_SHOW);
	if (m_bStatusBarWasVisible)
		m_wndStatusBar.ShowWindow (SW_SHOW);
	MoveWindow (&m_mainRect);
	RecalcLayout ();
	if (m_bMax)
    ShowWindow (SW_SHOWMAXIMIZED);
	CMDIChildWnd *pChild = MDIGetActive ();
	// pchild can be NULL if the USER closed all the
	// childs during Full Screen Mode:
	if(pChild)
    {
		  if(m_bChildMax)
			  MDIMaximize (pChild);
		  else
        MDIRestore (pChild);
	  }
	m_bFullScreen = false;
}

int CMainFrame2::MDIGetCount ()
{
  ASSERT (::IsWindow (m_hWnd));
  int nCount = 0;
  CWnd * pWnd = GetWindow (GW_CHILD);
  ASSERT (pWnd);
  pWnd = pWnd->GetWindow (GW_CHILD);
  while (pWnd)
    {
      nCount++;
      pWnd = pWnd->GetWindow (GW_HWNDNEXT);
    }
  return nCount;
}

BOOL CMainFrame2::
PreCreateWindow (CREATESTRUCT & cs)
{
  CReg reg;
  if (reg.Open (HKEY_CURRENT_USER, REG_EDITPAD_MULTI, KEY_READ))
    {
      // reg.LoadNumber (_T ("ShowFlags"), (DWORD*) &status.flags);
      reg.LoadNumber (_T ("ShowCmd"), (DWORD*) &theApp.m_nCmdShow);

      if (reg.LoadNumber (_T ("Left"), (DWORD*) &cs.x) &&
        reg.LoadNumber (_T ("Right"), (DWORD*) &cs.cx))
        {
          cs.cx -= cs.x;
        }
      if (reg.LoadNumber (_T ("Top"), (DWORD*) &cs.y) &&
        reg.LoadNumber (_T ("Bottom"), (DWORD*) &cs.cy))
        {
          cs.cy -= cs.y;
        }
    }

  if (!CMDIFrameWnd::PreCreateWindow (cs))
    return FALSE;

  return TRUE;
}

void CMainFrame2::
ChooseStatusBar (BOOL bActivate)
{
  if (m_bActivated != bActivate)
    {
      if (bActivate)
        m_wndStatusBar.SetIndicators (indicators1, sizeof (indicators1) / sizeof (UINT));
      else
        m_wndStatusBar.SetIndicators (indicators, sizeof (indicators) / sizeof (UINT));
      m_bActivated = bActivate;
    }
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame2 diagnostics

#ifdef _DEBUG
void CMainFrame2::
AssertValid ()
const
{
  CMDIFrameWnd::AssertValid ();
}

void CMainFrame2::Dump (CDumpContext & dc)
const
{
  CMDIFrameWnd::Dump (dc);
}

#endif                          //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame2 message handlers

int CMainFrame2::
OnCreate (LPCREATESTRUCT lpCreateStruct)
{
  if (CMDIFrameWnd::OnCreate (lpCreateStruct) == -1)
    return -1;

  if (!m_wndToolBar.CreateEx (this, TBSTYLE_FLAT, WS_CHILD|WS_VISIBLE|CBRS_TOP|CBRS_GRIPPER|CBRS_TOOLTIPS|CBRS_FLYBY|CBRS_SIZE_DYNAMIC) ||
    !m_wndToolBar.LoadToolBar (IDR_MAINFRAME))
  {
    TRACE0("Failed to create toolbar\n");
    return -1;      // fail to create
  }

  if (!m_wndStatusBar.Create (this) ||
    !m_wndStatusBar.SetIndicators (indicators, sizeof (indicators) / sizeof (UINT)))
    {
      TRACE0 ("Failed to create status bar\n");
      return -1;                // fail to create

    }

  m_bActivated = FALSE;

  CString strMainToolbarTitle;
  strMainToolbarTitle.LoadString (IDS_MAIN_TOOLBAR);
  m_wndToolBar.SetWindowText (strMainToolbarTitle);
  m_wndToolBar.EnableDocking (CBRS_ALIGN_ANY);
  EnableDocking (CBRS_ALIGN_ANY);
  DockControlBar (&m_wndToolBar);

  CString strControlBarRegEntry = REGISTRY_ROOT;
  strControlBarRegEntry += CONTROL_BARS;

  CDockState dockState;
  dockState.LoadState (CONTROL_BARS);

	for (int i = 0; i < dockState.m_arrBarInfo.GetSize (); i ++)
	{
		CControlBarInfo* pInfo = (CControlBarInfo*) dockState.m_arrBarInfo [i];
		ASSERT (pInfo != NULL);

		if (!pInfo->m_bFloating)
		{
			if (GetControlBar (pInfo->m_nBarID) == NULL)
			{
				TRACE (_T("ControlBar %x is not valid!\n"), pInfo->m_nBarID);
				return 0;
			}
		}
	}
  SetDockState (dockState);

  return 0;
}

void CMainFrame2::OnClose()
{
  theApp.SaveLastDocs ();
  if (m_bFullScreen)
    FullScreenOff ();
  WINDOWPLACEMENT status;
  status.length = sizeof (WINDOWPLACEMENT);
  GetWindowPlacement (&status);

  CReg reg;
  if (reg.Create (HKEY_CURRENT_USER, REG_EDITPAD_MULTI, KEY_WRITE))
    {
      VERIFY (reg.SaveNumber (_T ("ShowFlags"), status.flags));
      VERIFY (reg.SaveNumber (_T ("ShowCmd"), status.showCmd));

      VERIFY (reg.SaveNumber (_T ("Left"), status.rcNormalPosition.left));
      VERIFY (reg.SaveNumber (_T ("Top"), status.rcNormalPosition.top));
      VERIFY (reg.SaveNumber (_T ("Right"), status.rcNormalPosition.right));
      VERIFY (reg.SaveNumber (_T ("Bottom"), status.rcNormalPosition.bottom));
    }

  CString strControlBarRegEntry = REGISTRY_ROOT;
  strControlBarRegEntry += CONTROL_BARS;

  CDockState dockState;
  GetDockState (dockState);
  dockState.SaveState (CONTROL_BARS);

  CMDIFrameWnd::OnClose();
}

void CMainFrame2::OnViewFullScreen()
{
	if (m_bFullScreen)
		FullScreenOff ();
	else
    FullScreenOn ();
}

void CMainFrame2::OnUpdateViewFullScreen(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck (m_bFullScreen);	
}

void CMainFrame2::OnFileCloseAll()
{
  if (theApp.SaveAllModified ())
    theApp.CloseAllDocuments (FALSE);
}

void CMainFrame2::OnCloseWindow()
{
  MDIGetActive ()->DestroyWindow ();
}

void CMainFrame2::OnUpdateCloseWindow(CCmdUI* pCmdUI)
{
  pCmdUI->Enable (MDIGetActive () != NULL);
}

void CMainFrame2::OnNextWindow()
{
  MDINext ();
}

void CMainFrame2::OnUpdateNextWindow(CCmdUI* pCmdUI)
{
  pCmdUI->Enable (MDIGetCount () > 1);
}

void CMainFrame2::OnPrevWindow()
{
  MDIPrev ();
}

void CMainFrame2::OnUpdatePrevWindow(CCmdUI* pCmdUI)
{
  pCmdUI->Enable (MDIGetCount () > 1);
}

//Alarm flashing until the window became focus
void CMainFrame2::FlashUntilFocus ()
  {
	  m_bGotFocus = FALSE;

	  BOOL bInvert = FALSE;
	  while (!m_bGotFocus)
	    {		
		    FlashWindow (bInvert);
		    bInvert= !bInvert;
		    Sleep (500);
		    if (GetFocus() == this)
			    break;
	    }		
	  FlashWindow(FALSE);
  }

void CMainFrame2::OnSetFocus (CWnd* pOldWnd)
{
	CMDIFrameWnd::OnSetFocus (pOldWnd);	
	m_bGotFocus = TRUE;	
}

void CMainFrame2::OnEncoding(UINT nID)
{
  ASSERT_VALID (this);
  ASSERT (nID >= ID_ENCODING_1);
  ASSERT (nID < (UINT) (ID_ENCODING_1 + nCodeNames + 1));
  CCrystalTextBuffer::m_nDefaultEncoding = nID - ID_ENCODING_1 - 1;
}

void CMainFrame2::OnUpdateEncoding(CCmdUI* pCmdUI)
{
  CMenu *pMenu = pCmdUI->m_pMenu ? pCmdUI->m_pMenu->GetSubMenu (0) : NULL;

  if (!pMenu)
    return;

  for (int iEnc = 0; iEnc < 16; iEnc++)
    pMenu->DeleteMenu (pCmdUI->m_nID + iEnc, MF_BYCOMMAND);

  TCHAR buf[40];
  LPCTSTR *pszCodeName = pszCodeNames;
  iEnc = 0;
  _stprintf (buf, _T("&%d %s"), iEnc++ % 10, _T ("No recoding"));
  pMenu->InsertMenu (pCmdUI->m_nIndex++, MF_STRING|MF_BYPOSITION|((DWORD) CCrystalTextBuffer::m_nDefaultEncoding == pCmdUI->m_nID - ID_ENCODING_1 - 1 ? MF_CHECKED : 0), pCmdUI->m_nID, buf);
  pCmdUI->m_nID++;
  do
    {
      _stprintf (buf, _T("&%d %s"), iEnc++ % 10, *pszCodeName++);
      pMenu->InsertMenu (pCmdUI->m_nIndex++, MF_STRING|MF_BYPOSITION|((DWORD) CCrystalTextBuffer::m_nDefaultEncoding == pCmdUI->m_nID - ID_ENCODING_1 - 1 ? MF_CHECKED : 0), pCmdUI->m_nID, buf);
      pCmdUI->m_nID++;
    }
  while (*pszCodeName);

  pCmdUI->m_nIndex--;
  pCmdUI->m_nIndexMax = pMenu->GetMenuItemCount();

  pCmdUI->m_bEnableChanged = TRUE;
}
