///////////////////////////////////////////////////////////////////////////
//  File:    editpad.cpp
//  Version: 1.1.0.4
//  Updated: 19-Jul-1998
//
//  Copyright:  Ferdinand Prantl, portions by Stcherbatchenko Andrei
//  E-mail:     prantl@ff.cuni.cz
//
//  Application core
//
//  You are free to use or modify this code to the following restrictions:
//  - Acknowledge me somewhere in your about box, simple "Parts of code by.."
//  will be enough. If you can't (or don't want to), contact me personally.
//  - LEAVE THIS HEADER INTACT
////////////////////////////////////////////////////////////////////////////

// editpad.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "editpad.h"

#include "mainfrm.h"
#include "editpaddoc.h"
#include "editpadview.h"
#include "epricdoc.h"
#include "epricvw.h"
#include "epbindoc.h"
#include "epbinvw.h"
#include "editcmd.h"
#include "memcombo.h"
#include "registry.h"
#include "editreg.h"
#include "splash.h"
#include "tipdlg.h"
#include "aboutdlg.h"
#include "edfopdlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEditPadApp

BEGIN_MESSAGE_MAP (CEditPadApp, CWinApp)
//{{AFX_MSG_MAP(CEditPadApp)
ON_COMMAND (ID_APP_ABOUT, OnAppAbout)
ON_COMMAND(ID_HELP_TIPOFTHEDAY, OnTipOfTheDay)
ON_UPDATE_COMMAND_UI (ID_TOOLS_SPELLING_SET, OnUpdateToolsSpellingSet)
ON_COMMAND (ID_TOOLS_SPELLING_SET, OnToolsSpellingSet)
//}}AFX_MSG_MAP
// Standard file based document commands
ON_COMMAND (ID_FILE_NEW, OnFileNew)
ON_COMMAND (ID_FILE_OPEN, OnFileOpen)
// Standard print setup command
ON_COMMAND (ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP ()

/////////////////////////////////////////////////////////////////////////////
// CEditPadApp construction

CEditPadApp::CEditPadApp () : m_dwFlags (EP_SAVE_SETS_ON_EXIT|EP_NOTIFY_CHANGES|EP_ADVANCED_UI|EP_REMEMBER_LASTPOS)
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

DWORD CEditPadApp::
GetFlags ()
{
	return m_dwFlags;
}

void CEditPadApp::
SetFlags (DWORD dwFlags)
{
	if (m_dwFlags != dwFlags)
		{
			m_dwFlags = dwFlags;
		}
}

void CEditPadApp::
LoadSettings ()
{
  CEditPadView::LoadSettings ();
  CMemComboBox::LoadSettings ();
  *CEditPadView::szWIspellPath = _T ('\0');
  CReg reg;
  if (reg.Open (HKEY_CURRENT_USER, REG_EDITPAD_SINGLE, KEY_READ))
    {
      reg.LoadString (_T ("LastFile"), m_sLastDoc);
      reg.LoadNumber (_T ("Flags"), &m_dwFlags);
    }
  reg.Close ();
  if (reg.Open (HKEY_CURRENT_USER, REG_EDITPAD, KEY_READ))
    {
      reg.LoadString (_T ("WIspellPath"), CEditPadView::szWIspellPath, _MAX_PATH);
    }
}

void CEditPadApp::
LoadLastDoc ()
{
	if (!m_sLastDoc.IsEmpty ())
		{
			OpenDocumentFile (m_sLastDoc);
			m_sLastDoc.Empty ();
		}
}

void CEditPadApp::
SaveSettings ()
{
	CEditPadView::SaveSettings ();
	CMemComboBox::SaveSettings ();
	CReg reg;
	if (reg.Create (HKEY_CURRENT_USER, REG_EDITPAD_SINGLE, KEY_WRITE))
		{
      VERIFY (reg.SaveString (_T ("LastFile"), m_sLastDoc));
			VERIFY (reg.SaveNumber (_T ("Flags"), m_dwFlags));
		}
  reg.Close ();
  if (reg.Create (HKEY_CURRENT_USER, REG_EDITPAD, KEY_WRITE))
    {
      VERIFY (reg.SaveString (_T ("WIspellPath"), CEditPadView::szWIspellPath));
    }
  reg.Close ();
  struct
    {
      TCHAR path [_MAX_PATH];
      time_t stamp;
    } oOdd[REMEMBERED_TO_REMOVE];
  time_t stamp;
  time (&stamp);
  stamp += 1000;
  int i;
  for (i = 0; i < REMEMBERED_TO_REMOVE; i++)
    oOdd[i].stamp = stamp;
  CString sKey = REG_EDITPAD;
  sKey += _T ("\\Remembered");
  DWORD nEntries;
  LPCTSTR pszPath;
  RegVal vValue;
  if (reg.Open (HKEY_CURRENT_USER, sKey, KEY_READ|KEY_WRITE) && reg.HasEntries (NULL, &nEntries) && nEntries > MAX_REMEMBERED && reg.FindFirstValue (pszPath, &vValue))
    {
      DWORD dwLastPos[3];
      int nDelta, nMaxDelta, nPos;
      do
        {
          if (RegValGetBinary (&vValue, (LPBYTE) dwLastPos, sizeof (dwLastPos)))
            {
              nMaxDelta = 0;
              nPos = -1;
              for (i = 0; i < REMEMBERED_TO_REMOVE; i++)
                {
                  nDelta = oOdd[i].stamp - *dwLastPos;
                  if (nDelta > nMaxDelta)
                    {
                      nMaxDelta = nDelta;
                      nPos = i;
                    }
                }
              if (nPos >= 0)
                {
                  _tcscpy (oOdd[nPos].path, pszPath);
                  oOdd[nPos].stamp = *dwLastPos;
                }
            }
        }
      while (reg.FindNextValue (pszPath, &vValue));
      reg.FindClose ();
      for (i = 0; i < REMEMBERED_TO_REMOVE; i++)
        reg.DeleteKey (oOdd[i].path);
    }
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CEditPadApp object

CEditPadApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CEditPadApp initialization

BOOL CEditPadApp::
InitInstance ()
{
	CReg reg;
	if (reg.Open (HKEY_CURRENT_USER, REG_EDITPAD_SINGLE, KEY_READ|KEY_WRITE))
		{
      DWORD dwVersion;
      if (!reg.LoadNumber (NULL, _T ("Version"), &dwVersion) || dwVersion < REG_EDITPAD_VERSION)
        {
          reg.DeleteSubKeys ();
          VERIFY (reg.SaveNumber (_T ("Version"), (DWORD) REG_EDITPAD_VERSION));
	      }
    }
  reg.Close ();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

	// Initialize the common controls library
	/*INITCOMMONCONTROLSEX icex;
	icex.dwSize = sizeof( icex );
	icex.dwICC = ICC_USEREX_CLASSES;
	VERIFY( InitCommonControlsEx( &icex ) );*/

  free ((void*) m_pszHelpFilePath);
  m_pszHelpFilePath = _tcsdup (_T ("editpads.hlp"));

	// initialized OLE 2.0 libraries
	if (!AfxOleInit ())
		{
			AfxMessageBox (IDS_OLE_INIT_FAILED);
			return FALSE;
		}

#ifdef _AFXDLL
	Enable3dControls ();          // Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic ();    // Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey (_T ("EditPad\\Single"));

	LoadStdProfileSettings (16);  // Load standard INI file options (including MRU)
	CMemComboBox::LoadSettings ();
	LoadSettings ();

  // Parse command line for standard shell commands, DDE, file open
  CCommandLineInfo cmdInfo;
  ParseCommandLine (cmdInfo);

	CSplashWnd splash;
	BOOL bSplash = cmdInfo.m_bShowSplash && (m_dwFlags & EP_NO_SPLASH) == 0;
	if (!cmdInfo.m_bRunEmbedded)
	{
		switch (m_nCmdShow)
		{
			case SW_HIDE:
			case SW_SHOWMINIMIZED:
			case SW_MINIMIZE:
			case SW_SHOWMINNOACTIVE:
				bSplash = FALSE;
		}
	}
	else
	{
		//Excel 4 will start OLE servers minimized
		m_nCmdShow = SW_SHOWNORMAL;
	}
	if (bSplash)
	{
		// only show splash if not embedded
		splash.Create(NULL);
		splash.ShowWindow(SW_SHOW);
		splash.UpdateWindow();
	}

	// Initialize RichEdit control
	if (LoadLibrary(_T("RICHED32.DLL")) == NULL)
	{
		AfxMessageBox(IDS_RICHED_LOAD_FAIL, MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate *pDocTemplate;
	if (m_dwFlags & EP_ADVANCED_UI)
  	pDocTemplate = new CSingleDocTemplate (
  									 IDR_MAINFRAME,
  									 RUNTIME_CLASS (CEditPadDoc),
  									 RUNTIME_CLASS (CMainFrame),   // main SDI frame window
  									 RUNTIME_CLASS (CEditPadView));
  else
  	pDocTemplate = new CSingleDocTemplate (
  									 IDR_MAINFRAME,
  									 RUNTIME_CLASS (CEditPadDoc),
  									 RUNTIME_CLASS (CMainFrame2),   // main SDI frame window
  									 RUNTIME_CLASS (CEditPadView));
	AddDocTemplate (pDocTemplate);

	// Enable DDE Execute open
	//  EnableShellOpen();
	//  RegisterShellFileTypes(TRUE);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand (cmdInfo))
		return FALSE;

	POSITION pos = pDocTemplate->GetFirstDocPosition ();
	ASSERT (pos);
	CDocument *pDoc = pDocTemplate->GetNextDoc (pos);
	if ((m_dwFlags & EP_OPEN_LAST_DOC) && (!pDoc || pDoc->GetPathName ().IsEmpty ()))
		{
			LoadLastDoc ();
		}

	/*if (m_dwFlags & EP_ADVANCED_UI)
  	pDocTemplate = new CSingleDocTemplate (
  									 IDR_EDITPADRICHTYPE,
  									 RUNTIME_CLASS (CEditPadRichDoc),
  									 RUNTIME_CLASS (CMainFrame),   // main SDI frame window
  									 RUNTIME_CLASS (CEditPadRichView));
  else
  	pDocTemplate = new CSingleDocTemplate (
  									 IDR_EDITPADRICHTYPE,
  									 RUNTIME_CLASS (CEditPadRichDoc),
  									 RUNTIME_CLASS (CMainFrame2),   // main SDI frame window
  									 RUNTIME_CLASS (CEditPadView));
	AddDocTemplate (pDocTemplate);
	if (m_dwFlags & EP_ADVANCED_UI)
  	pDocTemplate = new CSingleDocTemplate (
  									 IDR_EDITPADBINTYPE,
  									 RUNTIME_CLASS (CEditPadBinDoc),
  									 RUNTIME_CLASS (CMainFrame),   // main SDI frame window
  									 RUNTIME_CLASS (CEditPadBinView));
  else
  	pDocTemplate = new CSingleDocTemplate (
  									 IDR_EDITPADBINTYPE,
  									 RUNTIME_CLASS (CEditPadBinDoc),
  									 RUNTIME_CLASS (CMainFrame2),   // main SDI frame window
  									 RUNTIME_CLASS (CEditPadBinView));
	AddDocTemplate (pDocTemplate);*/

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles ();

  if (m_dwFlags & EP_ADVANCED_UI)
    {
      LoadCustomState ();
    }

	// The one and only window has been initialized, so show and update it.
  DWORD dwCmdShow;
  if (reg.Open (HKEY_CURRENT_USER, REG_EDITPAD_SINGLE, KEY_READ))
    {
      reg.LoadNumber (_T ("ShowCmd"), (DWORD*) &dwCmdShow);
    }
  else
    dwCmdShow = SW_SHOW;
	m_pMainWnd->ShowWindow ((m_dwFlags & EP_MAXIMIZE_WND) ? SW_SHOWMAXIMIZED : dwCmdShow);
	m_pMainWnd->UpdateWindow ();

  if (bSplash)
    splash.DestroyWindow();
  CTipDlg dlg;
  if(dlg.m_bStartup)
    dlg.DoModal();

	return TRUE;
}

BOOL CEditPadApp::
ExitInstance ()
{
  FreeLibrary (GetModuleHandle (_T ("RICHED32.DLL")));
	BOOL result = CWinApp::ExitInstance ();
	if (m_dwFlags & EP_SAVE_SETS_ON_EXIT)
		{
			SaveSettings ();
		}
	return result;
}

void CEditPadApp::ParseCommandLine(CCommandLineInfo &rCmdInfo) //  by Serge Weinstock
	{
		// notepad receive only one argument, so we have to merge all the args
		m_OneArg.Empty();
		for (int i = 1; i < __argc; i++)
		  {
			  if (i > 1)
          m_OneArg += _T(' ');
			  m_OneArg += __targv[i];
		  }
		rCmdInfo.ParseParam(m_OneArg, false, true);
	}

/////////////////////////////////////////////////////////////////////////////
// Customization load/save methods

#define KEYBOARD_ENTRY _T("Keyboard")
#define MOUSE_ENTRY _T("Mouse")
#define MENU_ENTRY _T("Menus")

void CEditPadApp::LoadCustomState ()
{
  if (m_dwFlags & EP_ADVANCED_UI)
    {
      CString strKbdEntry = REGISTRY_ROOT;
      strKbdEntry += KEYBOARD_ENTRY;
      
      m_KeyboardManager.LoadState (strKbdEntry);
      
      CString strMenuEntry = REGISTRY_ROOT;
      strMenuEntry += MENU_ENTRY;
      
      // m_ContextMenuManager.AddMenu (_T("Document Menu"), IDR_EDITPADTYPE);
      // m_ContextMenuManager.AddMenu (_T("RTF Menu"), IDR_EDITPADRICHTYPE);
      // m_ContextMenuManager.AddMenu (_T("Binary Menu"), IDR_EDITPADBINTYPE);
      m_ContextMenuManager.AddMenu (_T("Context Menu"), IDR_CONTEXT_MENU);
      m_ContextMenuManager.AddMenu (_T("Toolbar Menu"), IDR_POPUP_TOOLBAR);
      
      m_ContextMenuManager.LoadState (strMenuEntry);
    }
}

void CEditPadApp::SaveCustomState ()
{
  CWnd *pWnd = AfxGetMainWnd ();
  if(pWnd->IsKindOf (RUNTIME_CLASS (CMainFrame)))
    {
      CString strKbdEntry = REGISTRY_ROOT;
      strKbdEntry += KEYBOARD_ENTRY;
      
      m_KeyboardManager.SaveState (strKbdEntry);
      
      CString strMenuEntry = REGISTRY_ROOT;
      strMenuEntry += MENU_ENTRY;
      
      m_ContextMenuManager.SaveState (strMenuEntry);
    }
}

BOOL CEditPadApp::ShowPopupMenu (UINT uiMenuResId, const CPoint &point, CWnd *pWnd)
{
  if (m_dwFlags & EP_ADVANCED_UI)
    {
      return m_ContextMenuManager.ShowPopupMenu (uiMenuResId, point.x, point.y, pWnd);
    }
  else
    {
      return FALSE;
    }
}

/////////////////////////////////////////////////////////////////////////////
// CEditPadApp message handlers

class CPubDocManager : public CObject
{
	DECLARE_DYNAMIC(CDocManager)
public:

// Constructor
	CDocManager();

	//Document functions
	virtual void AddDocTemplate(CDocTemplate* pTemplate) = 0;
	virtual POSITION GetFirstDocTemplatePosition() const = 0;
	virtual CDocTemplate* GetNextDocTemplate(POSITION& pos) const = 0;
	virtual void RegisterShellFileTypes(BOOL bCompat) = 0;
	// void UnregisterShellFileTypes();
	virtual CDocument* OpenDocumentFile(LPCTSTR lpszFileName) = 0; // open named file
	virtual BOOL SaveAllModified() = 0; // save before exit
	virtual void CloseAllDocuments(BOOL bEndSession) = 0; // close documents before exiting
	virtual int GetOpenDocumentCount() = 0;

	// helper for standard commdlg dialogs
	virtual BOOL DoPromptFileName(CString& fileName, UINT nIDSTitle,
			DWORD lFlags, BOOL bOpenFileDialog, CDocTemplate* pTemplate) = 0;

//Commands
	// Advanced: process async DDE request
	virtual BOOL OnDDECommand(LPTSTR lpszCommand) = 0;
	virtual void OnFileNew() = 0;
	virtual void OnFileOpen() = 0;

// Implementation
	CPtrList m_templateList;
	// int GetDocumentCount(); // helper to count number of total documents

	// static CPtrList* pStaticList;       // for static CDocTemplate objects
	// static BOOL bStaticInit;            // TRUE during static initialization
	// static CDocManager* pStaticDocManager;  // for static CDocTemplate objects

public:
	// virtual ~CDocManager();
#ifdef _DEBUG
	virtual void AssertValid() const = 0;
	virtual void Dump(CDumpContext& dc) const = 0;
#endif
};

void CEditPadApp::OnFileNew()
{
  if (m_pDocManager != NULL)
    {
      CDocTemplate *pBinTempl = (CDocTemplate*) ((CPubDocManager*) m_pDocManager)->m_templateList.GetTail ();
      if (((CPubDocManager*) m_pDocManager)->m_templateList.GetCount () > 1)
        {
          ((CPubDocManager*) m_pDocManager)->m_templateList.RemoveTail ();
          CWinApp::OnFileNew();
          ((CPubDocManager*) m_pDocManager)->m_templateList.AddTail (pBinTempl);
        }
      else
        CWinApp::OnFileNew();
    }
}

void CEditPadApp::OnFileOpen()
{
  ASSERT(m_pDocManager != NULL);
  CEditFileOpenDlg dlgFile (TRUE, _T (""), _T (""), OFN_FILEMUSTEXIST|OFN_EXPLORER|OFN_ENABLEHOOK, _T("Supported Documents (...)|*.lsp;*.bas;*.vb;*.vbs;*.frm;*.dsm;*.bat;*.btm;*.cmd;*.c;*.cc;*.cpp;*.cxx;*.h;*.hpp;*.hxx;*.hm;*.inl;*.rh;*.tlh;*.tli;*.xs;*.dcl;*.dcc;*.f;*.f90;*.f9p;*.fpp;*.html;*.htm;*.shtml;*.ihtml;*.ssi;*.rul;*.java;*.jav;*.pas;*.pl;*.txt;*.doc;*.diz;*.py;*.rc;*.dlg;*.rex;*.rexx;*.sgml;*.sh;*.conf;*.scm;*.sql;*.tcl;*.text;*.sty;*.cls;*.clo;*.ltx;*.fd;*.dtx|All Files (*.*)|*.*||"), AfxGetMainWnd ());
  CString title;
  VERIFY (title.LoadString (AFX_IDS_OPENFILE));

  /*CString strFilter;
  CString strDefault;
  if (pTemplate != NULL)
    {
      ASSERT_VALID(pTemplate);
      _AfxAppendFilterSuffix(strFilter, dlgFile.m_ofn, pTemplate, &strDefault);
    }
  else
    {
      // do for all doc template
      POSITION pos = m_templateList.GetHeadPosition();
      BOOL bFirst = TRUE;
      while (pos != NULL)
        {
          CDocTemplate* pTemplate = (CDocTemplate*)m_templateList.GetNext(pos);
          _AfxAppendFilterSuffix(strFilter, dlgFile.m_ofn, pTemplate,
          bFirst ? &strDefault : NULL);
          bFirst = FALSE;
        }
    }

  // append the "*.*" all files filter
  CString allFilter;
  VERIFY(allFilter.LoadString(AFX_IDS_ALLFILTER));
  strFilter += allFilter;
  strFilter += (TCHAR)'\0';   // next string please
  strFilter += _T("*.*");
  strFilter += (TCHAR)'\0';   // last string
  dlgFile.m_ofn.nMaxCustFilter++;

  dlgFile.m_ofn.lpstrFilter = strFilter;*/
  dlgFile.m_ofn.lpstrTitle = title;
  static TCHAR szPath[_MAX_PATH];
  dlgFile.m_ofn.lpstrInitialDir = szPath;

  if (dlgFile.DoModal() == IDOK)
    {
      CEditPadDoc *pDoc;
      int oldEncoding = CCrystalTextBuffer::m_nDefaultEncoding;
      CCrystalTextBuffer::m_nDefaultEncoding = dlgFile.m_nEncoding;
      pDoc = (CEditPadDoc*) OpenDocumentFileEx (dlgFile.GetPathName (), dlgFile.m_nFormat);
      if (pDoc && dlgFile.GetReadOnlyPref())
        pDoc->OnReadOnly ();
      CCrystalTextBuffer::m_nDefaultEncoding = oldEncoding;
    }
}

void CEditPadApp::OnAppAbout ()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal ();
}

void CEditPadApp::OnTipOfTheDay ()
{
	CTipDlg dlg;
	dlg.DoModal();
}

#include <..\src\afximpl.h>

CDocument *CEditPadApp::OpenDocumentFile(LPCTSTR lpszFileName)
{
  return CWinApp::OpenDocumentFile(lpszFileName);
}

CDocument *CEditPadApp::OpenDocumentFileEx(LPCTSTR lpszFileName, int nType)
{
	TCHAR szPath[_MAX_PATH];
	ASSERT (_tcslen (lpszFileName) < countof (szPath));
	TCHAR szTemp[_MAX_PATH];
	if (lpszFileName[0] == '\"')
		++lpszFileName;
	_tcsncpy (szTemp, lpszFileName, _MAX_PATH);
	LPTSTR lpszLast = _tcsrchr (szTemp, '\"');
	if (lpszLast != NULL)
		*lpszLast = 0;
	AfxFullPath (szPath, szTemp);
	TCHAR szLinkName[_MAX_PATH];
	if (AfxResolveShortcut (AfxGetMainWnd (), szPath, szLinkName, _MAX_PATH))
		_tcscpy (szPath, szLinkName);

  POSITION pos = GetFirstDocTemplatePosition ();
  CDocTemplate *pTempl, *pBestTemplate = NULL;
	CDocument *pOpenDocument = NULL;
  CDocTemplate::Confidence match;
  int nIndex = 0;

  if (nType == 1)
    nType = 0;
  while (pos)
    {
      pTempl = GetNextDocTemplate (pos);
  		ASSERT_KINDOF(CDocTemplate, pTempl);
		  ASSERT (pOpenDocument == NULL);
		  match = pTempl->MatchDocType (szPath, pOpenDocument);
      if (match == CDocTemplate::yesAlreadyOpen)
        break;
      if (nIndex++ == nType)
        pBestTemplate = pTempl;
    }
  ASSERT (pTempl);
  if (!pBestTemplate)
    pBestTemplate = pTempl;

	if (pOpenDocument != NULL)
	  {
		  POSITION pos = pOpenDocument->GetFirstViewPosition();
		  if (pos)
		    {
			    CView* pView = pOpenDocument->GetNextView(pos); // get first one
			    ASSERT_VALID(pView);
			    CFrameWnd* pFrame = pView->GetParentFrame();
			    if (pFrame != NULL)
				    pFrame->ActivateFrame();
			    else
				    TRACE0("Error: Can not find a frame for document to activate.\n");
			    CFrameWnd* pAppFrame;
			    if (pFrame != (pAppFrame = (CFrameWnd*)AfxGetApp()->m_pMainWnd))
			      {
				      ASSERT_KINDOF(CFrameWnd, pAppFrame);
				      pAppFrame->ActivateFrame();
			      }
		    }
		  else
		    {
			    TRACE0("Error: Can not find a view for document to activate.\n");
		    }
		  return pOpenDocument;
	  }

	if (pBestTemplate == NULL)
	  {
		  AfxMessageBox(AFX_IDP_FAILED_TO_OPEN_DOC);
		  return NULL;
	  }

	return pBestTemplate->OpenDocumentFile (szPath);
}

void CEditPadApp::
OnUpdateToolsSpellingSet (CCmdUI * pCmdUI)
{
  pCmdUI->Enable (CCrystalEditView::LoadSpellDll (FALSE));
}

void CEditPadApp::
OnToolsSpellingSet ()
{
  if (CCrystalEditView::LoadSpellDll ())
    {
      CCrystalEditView::spellData.hParent = AfxGetMainWnd ()->GetSafeHwnd ();
      CCrystalEditView::SpellConfig (&CCrystalEditView::spellData);
    }
}
